@@ -16,7 +16,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-//  $Id: panel.cxx,v 1.56 2002/02/05 14:57:46 curt Exp $
+//  $Id: panel.cxx,v 1.28 2004/02/23 09:03:41 ehofman Exp $
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -38,7 +38,6 @@
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 #include <Main/viewmgr.hxx>
-#include <Objects/texload.h>
 #include <Time/light.hxx>
 
 #include "hud.hxx"
@@ -49,16 +48,11 @@
 #define WIN_W 1024
 #define WIN_H 768
 
-#if defined( NONE ) && defined( _MSC_VER )
-#  pragma message( "A sloppy coder has defined NONE as a macro!!!" )
-#  undef NONE
-#elif defined( NONE )
-#  pragma warn A sloppy coder has defined NONE as a macro!!!
-#  undef NONE
-#endif
+// The number of polygon-offset "units" to place between layers.  In
+// principle, one is supposed to be enough.  In practice, I find that
+// my hardware/driver requires many more.
+#define POFF_UNITS 4
 
-
-
 ////////////////////////////////////////////////////////////////////////
 // Local functions.
 ////////////////////////////////////////////////////////////////////////
@@ -84,10 +78,15 @@ get_aspect_adjust (int xsize, int ysize)
 bool
 fgPanelVisible ()
 {
-  return ((current_panel != 0) &&
-	  (current_panel->getVisibility()) &&
-	  (globals->get_viewmgr()->get_current() == 0) &&
-	  (globals->get_current_view()->get_view_offset() == 0.0));
+     if(globals->get_current_panel() == 0)
+	return false;
+     if(globals->get_current_panel()->getVisibility() == 0)
+	return false;
+     if(globals->get_viewmgr()->get_current() != 0)
+	return false;
+     if(globals->get_current_view()->getHeadingOffset_deg() * SGD_DEGREES_TO_RADIANS != 0)
+	return false;
+     return true;
 }
 
 
@@ -162,9 +161,9 @@ FGCroppedTexture::getTexture ()
 // Implementation of FGPanel.
 ////////////////////////////////////////////////////////////////////////
 
-FGPanel * current_panel = NULL;
 static fntRenderer text_renderer;
-
+static fntTexFont *default_font = 0;
+static fntTexFont *led_font = 0;
 
 /**
  * Constructor.
@@ -173,13 +172,15 @@ FGPanel::FGPanel ()
   : _mouseDown(false),
     _mouseInstrument(0),
     _width(WIN_W), _height(int(WIN_H * 0.5768 + 1)),
-    _x_offset(0), _y_offset(0), _view_height(int(WIN_H * 0.4232)),
-    _bound(false),
-    _jitter(0.0),
+    _view_height(int(WIN_H * 0.4232)),
+    _visibility(fgGetNode("/sim/panel/visibility", true)),
+    _x_offset(fgGetNode("/sim/panel/x-offset", true)),
+    _y_offset(fgGetNode("/sim/panel/y-offset", true)),
+    _jitter(fgGetNode("/sim/panel/jitter", true)),
+    _flipx(fgGetNode("/sim/panel/flip-x", true)),
     _xsize_node(fgGetNode("/sim/startup/xsize", true)),
     _ysize_node(fgGetNode("/sim/startup/ysize", true))
 {
-  setVisibility(fgPanelVisible());
 }
 
 
@@ -188,8 +189,6 @@ FGPanel::FGPanel ()
  */
 FGPanel::~FGPanel ()
 {
-  if (_bound)
-    unbind();
   for (instrument_list_type::iterator it = _instruments.begin();
        it != _instruments.end();
        it++) {
@@ -215,7 +214,28 @@ FGPanel::addInstrument (FGPanelInstrument * instrument)
 void
 FGPanel::init ()
 {
-  // NO-OP
+    SGPath base_path;
+    char* envp = ::getenv( "FG_FONTS" );
+    if ( envp != NULL ) {
+        base_path.set( envp );
+    } else {
+        base_path.set( globals->get_fg_root() );
+	base_path.append( "Fonts" );
+    }
+
+    SGPath fntpath;
+
+    // Install the default font
+    fntpath = base_path;
+    fntpath.append( "typewriter.txf" );
+    default_font = new fntTexFont ;
+    default_font -> load ( (char *)fntpath.c_str() ) ;
+
+    // Install the LED font
+    fntpath = base_path;
+    fntpath.append( "led.txf" );
+    led_font = new fntTexFont ;
+    led_font -> load ( (char *)fntpath.c_str() ) ;
 }
 
 
@@ -225,15 +245,10 @@ FGPanel::init ()
 void
 FGPanel::bind ()
 {
-  fgTie("/sim/panel/visibility", &_visibility);
   fgSetArchivable("/sim/panel/visibility");
-  fgTie("/sim/panel/x-offset", &_x_offset);
   fgSetArchivable("/sim/panel/x-offset");
-  fgTie("/sim/panel/y-offset", &_y_offset);
   fgSetArchivable("/sim/panel/y-offset");
-  fgTie("/sim/panel/jitter", &_jitter);
   fgSetArchivable("/sim/panel/jitter");
-  _bound = true;
 }
 
 
@@ -243,10 +258,6 @@ FGPanel::bind ()
 void
 FGPanel::unbind ()
 {
-  fgUntie("/sim/panel/visibility");
-  fgUntie("/sim/panel/x-offset");
-  fgUntie("/sim/panel/y-offset");
-  _bound = false;
 }
 
 
@@ -254,21 +265,14 @@ FGPanel::unbind ()
  * Update the panel.
  */
 void
-FGPanel::update (int dt)
+FGPanel::update (double dt)
 {
 				// Do nothing if the panel isn't visible.
     if ( !fgPanelVisible() ) {
         return;
     }
 
-				// If the mouse is down, do something
-    if (_mouseDown) {
-        _mouseDelay--;
-        if (_mouseDelay < 0) {
-            _mouseInstrument->doMouseAction(_mouseButton, _mouseX, _mouseY);
-            _mouseDelay = 2;
-        }
-    }
+    updateMouseDelay();
 
 				// Now, draw the panel
     float aspect_adjust = get_aspect_adjust(_xsize_node->getIntValue(),
@@ -279,6 +283,22 @@ FGPanel::update (int dt)
         update(WIN_X, WIN_W, WIN_Y, int(WIN_H / aspect_adjust));
 }
 
+/**
+ * Handle repeatable mouse events.  Called from update() and from
+ * fgUpdate3DPanels().  This functionality needs to move into the
+ * input subsystem.  Counting a tick every two frames is clumsy...
+ */
+void FGPanel::updateMouseDelay()
+{
+    if (_mouseDown) {
+        _mouseDelay--;
+        if (_mouseDelay < 0) {
+            _mouseInstrument->doMouseAction(_mouseButton, 0, _mouseX, _mouseY);
+            _mouseDelay = 2;
+        }
+    }
+}
+
 
 void
 FGPanel::update (GLfloat winx, GLfloat winw, GLfloat winy, GLfloat winh)
@@ -288,89 +308,157 @@ FGPanel::update (GLfloat winx, GLfloat winw, GLfloat winy, GLfloat winh)
 				// The factors and bounds are just
 				// initial guesses; using sqrt smooths
 				// out the spikes.
-  double x_offset = _x_offset;
-  double y_offset = _y_offset;
+  double x_offset = _x_offset->getIntValue();
+  double y_offset = _y_offset->getIntValue();
 
-  if (_jitter != 0.0) {
+#if 0
+  if (_jitter->getFloatValue() != 0.0) {
     double a_x_pilot = current_aircraft.fdm_state->get_A_X_pilot();
     double a_y_pilot = current_aircraft.fdm_state->get_A_Y_pilot();
     double a_z_pilot = current_aircraft.fdm_state->get_A_Z_pilot();
 
     double a_zx_pilot = a_z_pilot - a_x_pilot;
     
-    int x_adjust = int(sqrt(fabs(a_y_pilot) * _jitter)) *
+    int x_adjust = int(sqrt(fabs(a_y_pilot) * _jitter->getFloatValue())) *
 		   (a_y_pilot < 0 ? -1 : 1);
-    int y_adjust = int(sqrt(fabs(a_zx_pilot) * _jitter)) *
+    int y_adjust = int(sqrt(fabs(a_zx_pilot) * _jitter->getFloatValue())) *
 		   (a_zx_pilot < 0 ? -1 : 1);
 
 				// adjustments in screen coordinates
     x_offset += x_adjust;
     y_offset += y_adjust;
   }
+#endif
 
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
-  gluOrtho2D(winx, winx + winw, winy, winy + winh); /* right side up */
-  // gluOrtho2D(winx + winw, winx, winy + winh, winy); /* up side down */
-
+  if ( _flipx->getBoolValue() ) {
+    gluOrtho2D(winx + winw, winx, winy + winh, winy); /* up side down */
+  } else {
+    gluOrtho2D(winx, winx + winw, winy, winy + winh); /* right side up */
+  }
+  
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   glLoadIdentity();
-
+  
   glTranslated(x_offset, y_offset, 0);
+  
+  draw();
 
-				// Draw the background
+  glMatrixMode(GL_PROJECTION);
+  glPopMatrix();
+  glMatrixMode(GL_MODELVIEW);
+  glPopMatrix();
+
+  ssgForceBasicState();
+  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+}
+
+void
+FGPanel::draw()
+{
+  // In 3D mode, it's possible that we are being drawn exactly on top
+  // of an existing polygon.  Use an offset to prevent z-fighting.  In
+  // 2D mode, this is a no-op.
+  glEnable(GL_POLYGON_OFFSET_FILL);
+  glPolygonOffset(-1, -POFF_UNITS);
+
+  // save some state
+  glPushAttrib( GL_COLOR_BUFFER_BIT | GL_ENABLE_BIT | GL_LIGHTING_BIT
+                | GL_TEXTURE_BIT | GL_PIXEL_MODE_BIT | GL_CULL_FACE 
+                | GL_DEPTH_BUFFER_BIT );
+
+  // Draw the background
   glEnable(GL_TEXTURE_2D);
   glDisable(GL_LIGHTING);
   glEnable(GL_BLEND);
   glEnable(GL_ALPHA_TEST);
   glEnable(GL_COLOR_MATERIAL);
-  // glColor4f(1.0, 1.0, 1.0, 1.0);
-  if ( cur_light_params.sun_angle * SGD_RADIANS_TO_DEGREES < 95.0 ) {
-      glColor4fv( cur_light_params.scene_diffuse );
-  } else {
-      glColor4f(0.7, 0.2, 0.2, 1.0);
-  }
-  glBindTexture(GL_TEXTURE_2D, _bg->getHandle());
-  // glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-  // glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-  glBegin(GL_POLYGON);
-  glTexCoord2f(0.0, 0.0); glVertex3f(WIN_X, WIN_Y, 0);
-  glTexCoord2f(1.0, 0.0); glVertex3f(WIN_X + _width, WIN_Y, 0);
-  glTexCoord2f(1.0, 1.0); glVertex3f(WIN_X + _width, WIN_Y + _height, 0);
-  glTexCoord2f(0.0, 1.0); glVertex3f(WIN_X, WIN_Y + _height, 0);
-  glEnd();
+  glEnable(GL_CULL_FACE);
+  glCullFace(GL_BACK);
+  glDisable(GL_DEPTH_TEST);
+  sgVec4 panel_color;
 
-				// Draw the instruments.
+  FGLight *l = (FGLight *)(globals->get_subsystem("lighting"));
+  sgCopyVec4( panel_color, l->scene_diffuse());
+  if ( fgGetDouble("/systems/electrical/outputs/instrument-lights") > 1.0 ) {
+      if ( panel_color[0] < 0.7 ) panel_color[0] = 0.7;
+      if ( panel_color[1] < 0.2 ) panel_color[1] = 0.2;
+      if ( panel_color[2] < 0.2 ) panel_color[2] = 0.2;
+  }
+  glColor4fv( panel_color );
+  if (_bg != 0) {
+    glBindTexture(GL_TEXTURE_2D, _bg->getHandle());
+    // glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+    // glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+    glBegin(GL_POLYGON);
+    glTexCoord2f(0.0, 0.0); glVertex2f(WIN_X, WIN_Y);
+    glTexCoord2f(1.0, 0.0); glVertex2f(WIN_X + _width, WIN_Y);
+    glTexCoord2f(1.0, 1.0); glVertex2f(WIN_X + _width, WIN_Y + _height);
+    glTexCoord2f(0.0, 1.0); glVertex2f(WIN_X, WIN_Y + _height);
+    glEnd();
+  } else {
+    for (int i = 0; i < 4; i ++) {
+      // top row of textures...(1,3,5,7)
+      glBindTexture(GL_TEXTURE_2D, _mbg[i*2]->getHandle());
+      glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+      glBegin(GL_POLYGON);
+      glTexCoord2f(0.0, 0.0); glVertex2f(WIN_X + (_width/4) * i, WIN_Y + (_height/2));
+      glTexCoord2f(1.0, 0.0); glVertex2f(WIN_X + (_width/4) * (i+1), WIN_Y + (_height/2));
+      glTexCoord2f(1.0, 1.0); glVertex2f(WIN_X + (_width/4) * (i+1), WIN_Y + _height);
+      glTexCoord2f(0.0, 1.0); glVertex2f(WIN_X + (_width/4) * i, WIN_Y + _height);
+      glEnd();
+      // bottom row of textures...(2,4,6,8)
+      glBindTexture(GL_TEXTURE_2D, _mbg[(i*2)+1]->getHandle());
+      glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+      glBegin(GL_POLYGON);
+      glTexCoord2f(0.0, 0.0); glVertex2f(WIN_X + (_width/4) * i, WIN_Y);
+      glTexCoord2f(1.0, 0.0); glVertex2f(WIN_X + (_width/4) * (i+1), WIN_Y);
+      glTexCoord2f(1.0, 1.0); glVertex2f(WIN_X + (_width/4) * (i+1), WIN_Y + (_height/2));
+      glTexCoord2f(0.0, 1.0); glVertex2f(WIN_X + (_width/4) * i, WIN_Y + (_height/2));
+      glEnd();
+    }
+  }
+
+  // Draw the instruments.
   instrument_list_type::const_iterator current = _instruments.begin();
   instrument_list_type::const_iterator end = _instruments.end();
 
   for ( ; current != end; current++) {
     FGPanelInstrument * instr = *current;
-    glLoadIdentity();
-    glTranslated(x_offset, y_offset, 0);
+    glPushMatrix();
     glTranslated(instr->getXPos(), instr->getYPos(), 0);
     instr->draw();
+    glPopMatrix();
   }
 
-  glMatrixMode(GL_PROJECTION);
-  glPopMatrix();
-  glMatrixMode(GL_MODELVIEW);
-  glPopMatrix();
-  ssgForceBasicState();
-  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-}
+  // Draw yellow "hotspots" if directed to.  This is a panel authoring
+  // feature; not intended to be high performance or to look good.
+  if ( fgGetBool("/sim/panel-hotspots") ) {
+    glDisable(GL_TEXTURE_2D);
+    glColor3f(1, 1, 0);
+    
+    for ( unsigned int i = 0; i < _instruments.size(); i++ )
+      _instruments[i]->drawHotspots();
+  }
 
 
+  // restore some original state
+  glPopAttrib();
+  glPolygonOffset(0, 0);
+  glDisable(GL_POLYGON_OFFSET_FILL);
+}
+
 /**
  * Set the panel's visibility.
  */
 void
 FGPanel::setVisibility (bool visibility)
 {
-  _visibility = visibility;
+  _visibility->setBoolValue( visibility );
 }
 
 
@@ -380,7 +468,7 @@ FGPanel::setVisibility (bool visibility)
 bool
 FGPanel::getVisibility () const
 {
-  return _visibility;
+  return _visibility->getBoolValue();
 }
 
 
@@ -393,6 +481,15 @@ FGPanel::setBackground (ssgTexture * texture)
   _bg = texture;
 }
 
+/**
+ * Set the panel's multiple background textures.
+ */
+void
+FGPanel::setMultiBackground (ssgTexture * texture, int idx)
+{
+  _bg = 0;
+  _mbg[idx] = texture;
+}
 
 /**
  * Set the panel's x-offset.
@@ -401,7 +498,7 @@ void
 FGPanel::setXOffset (int offset)
 {
   if (offset <= 0 && offset >= -_width + WIN_W)
-    _x_offset = offset;
+    _x_offset->setIntValue( offset );
 }
 
 
@@ -412,43 +509,27 @@ void
 FGPanel::setYOffset (int offset)
 {
   if (offset <= 0 && offset >= -_height)
-    _y_offset = offset;
+    _y_offset->setIntValue( offset );
 }
 
-
 /**
- * Perform a mouse action.
+ * Handle a mouse action in panel-local (not screen) coordinates.
+ * Used by the 3D panel code in Model/panelnode.cxx, in situations
+ * where the panel doesn't control its own screen location.
  */
 bool
-FGPanel::doMouseAction (int button, int updown, int x, int y)
+FGPanel::doLocalMouseAction(int button, int updown, int x, int y)
 {
-				// FIXME: this same code appears in update()
-  int xsize = _xsize_node->getIntValue();
-  int ysize = _ysize_node->getIntValue();
-  float aspect_adjust = get_aspect_adjust(xsize, ysize);
-
-				// Note a released button and return
-  // cerr << "Doing mouse action\n";
+  // Note a released button and return
   if (updown == 1) {
+    if (_mouseInstrument != 0)
+        _mouseInstrument->doMouseAction(_mouseButton, 1, _mouseX, _mouseY);
     _mouseDown = false;
     _mouseInstrument = 0;
-    return true;
+    return false;
   }
 
-				// Scale for the real window size.
-  if (aspect_adjust < 1.0) {
-    x = int(((float)x / xsize) * WIN_W * aspect_adjust);
-    y = int(WIN_H - ((float(y) / ysize) * WIN_H));
-  } else {
-    x = int(((float)x / xsize) * WIN_W);
-    y = int((WIN_H - ((float(y) / ysize) * WIN_H)) / aspect_adjust);
-  }
-
-				// Adjust for offsets.
-  x -= _x_offset;
-  y -= _y_offset;
-
-				// Search for a matching instrument.
+  // Search for a matching instrument.
   for (int i = 0; i < (int)_instruments.size(); i++) {
     FGPanelInstrument *inst = _instruments[i];
     int ix = inst->getXPos();
@@ -462,14 +543,43 @@ FGPanel::doMouseAction (int button, int updown, int x, int y)
       _mouseButton = button;
       _mouseX = x - ix;
       _mouseY = y - iy;
-				// Always do the action once.
-      _mouseInstrument->doMouseAction(_mouseButton, _mouseX, _mouseY);
-      return true;
+      // Always do the action once.
+      return _mouseInstrument->doMouseAction(_mouseButton, 0,
+                                             _mouseX, _mouseY);
     }
   }
   return false;
 }
 
+/**
+ * Perform a mouse action.
+ */
+bool
+FGPanel::doMouseAction (int button, int updown, int x, int y)
+{
+				// FIXME: this same code appears in update()
+  int xsize = _xsize_node->getIntValue();
+  int ysize = _ysize_node->getIntValue();
+  float aspect_adjust = get_aspect_adjust(xsize, ysize);
+
+				// Scale for the real window size.
+  if (aspect_adjust < 1.0) {
+    x = int(((float)x / xsize) * WIN_W * aspect_adjust);
+    y = int(WIN_H - ((float(y) / ysize) * WIN_H));
+  } else {
+    x = int(((float)x / xsize) * WIN_W);
+    y = int((WIN_H - ((float(y) / ysize) * WIN_H)) / aspect_adjust);
+  }
+
+				// Adjust for offsets.
+  x -= _x_offset->getIntValue();
+  y -= _y_offset->getIntValue();
+
+  // Having fixed up the coordinates, fall through to the local
+  // coordinate handler.
+  return doLocalMouseAction(button, updown, x, y);
+} 
+
 
 
 ////////////////////////////////////////////////////////////////////////.
@@ -480,11 +590,14 @@ FGPanelAction::FGPanelAction ()
 {
 }
 
-FGPanelAction::FGPanelAction (int button, int x, int y, int w, int h)
-  : _button(button), _x(x), _y(y), _w(w), _h(h)
+FGPanelAction::FGPanelAction (int button, int x, int y, int w, int h,
+                              bool repeatable)
+    : _button(button), _x(x), _y(y), _w(w), _h(h), _repeatable(repeatable)
 {
-  for (unsigned int i = 0; i < _bindings.size(); i++)
-    delete _bindings[i];
+  for (unsigned int i = 0; i < 2; i++) {
+      for (unsigned int j = 0; j < _bindings[i].size(); j++)
+          delete _bindings[i][j];
+  }
 }
 
 FGPanelAction::~FGPanelAction ()
@@ -492,19 +605,24 @@ FGPanelAction::~FGPanelAction ()
 }
 
 void
-FGPanelAction::addBinding (FGBinding * binding)
+FGPanelAction::addBinding (FGBinding * binding, int updown)
 {
-  _bindings.push_back(binding);
+  _bindings[updown].push_back(binding);
 }
 
-void
-FGPanelAction::doAction ()
+bool
+FGPanelAction::doAction (int updown)
 {
   if (test()) {
-    int nBindings = _bindings.size();
-    for (int i = 0; i < nBindings; i++) {
-      _bindings[i]->fire();
+    if ((updown != _last_state) || (updown == 0 && _repeatable)) {
+        int nBindings = _bindings[updown].size();
+        for (int i = 0; i < nBindings; i++)
+            _bindings[updown][i]->fire();
     }
+    _last_state = updown;
+    return true;
+  } else {
+    return false;
   }
 }
 
@@ -553,6 +671,25 @@ FGPanelInstrument::~FGPanelInstrument ()
   }
 }
 
+void
+FGPanelInstrument::drawHotspots()
+{
+  for ( unsigned int i = 0; i < _actions.size(); i++ ) {
+    FGPanelAction* a = _actions[i];
+    float x1 = getXPos() + a->getX();
+    float x2 = x1 + a->getWidth();
+    float y1 = getYPos() + a->getY();
+    float y2 = y1 + a->getHeight();
+
+    glBegin(GL_LINE_LOOP);
+    glVertex2f(x1, y1);
+    glVertex2f(x1, y2);
+    glVertex2f(x2, y2);
+    glVertex2f(x2, y1);
+    glEnd();
+  }
+}
+
 void
 FGPanelInstrument::setPosition (int x, int y)
 {
@@ -599,16 +736,15 @@ FGPanelInstrument::addAction (FGPanelAction * action)
 
 				// Coordinates relative to centre.
 bool
-FGPanelInstrument::doMouseAction (int button, int x, int y)
+FGPanelInstrument::doMouseAction (int button, int updown, int x, int y)
 {
   if (test()) {
     action_list_type::iterator it = _actions.begin();
     action_list_type::iterator last = _actions.end();
     for ( ; it != last; it++) {
-      if ((*it)->inArea(button, x, y)) {
-	(*it)->doAction();
-	return true;
-      }
+      if ((*it)->inArea(button, x, y) &&
+          (*it)->doAction(updown))
+        return true;
     }
   }
   return false;
@@ -636,13 +772,13 @@ FGLayeredInstrument::~FGLayeredInstrument ()
 void
 FGLayeredInstrument::draw ()
 {
-  if (test()) {
-    for (int i = 0; i < (int)_layers.size(); i++) {
-      glPushMatrix();
-      glTranslatef(0.0, 0.0, (i / 100.0) + 0.1);
-      _layers[i]->draw();
-      glPopMatrix();
-    }
+  if (!test())
+    return;
+  
+  for (int i = 0; i < (int)_layers.size(); i++) {
+    glPushMatrix();
+    _layers[i]->draw();
+    glPopMatrix();
   }
 }
 
@@ -705,11 +841,15 @@ FGInstrumentLayer::transform () const
     FGPanelTransformation *t = *it;
     if (t->test()) {
       float val = (t->node == 0 ? 0.0 : t->node->getFloatValue());
+
+      if (t->has_mod)
+          val = fmod(val, t->mod);
       if (val < t->min) {
 	val = t->min;
       } else if (val > t->max) {
 	val = t->max;
       }
+
       if(t->table==0) {
 	val = val * t->factor + t->offset;
       } else {
@@ -758,6 +898,7 @@ void
 FGGroupLayer::draw ()
 {
   if (test()) {
+    transform();
     int nLayers = _layers.size();
     for (int i = 0; i < nLayers; i++)
       _layers[i]->draw();
@@ -802,12 +943,16 @@ FGTexturedLayer::draw ()
     
 				// From Curt: turn on the panel
 				// lights after sundown.
-    if ( cur_light_params.sun_angle * SGD_RADIANS_TO_DEGREES < 95.0 ) {
-      glColor4fv( cur_light_params.scene_diffuse );
-    } else {
-      glColor4f(0.7, 0.2, 0.2, 1.0);
-    }
+    sgVec4 panel_color;
 
+    FGLight *l = (FGLight *)(globals->get_subsystem("lighting"));
+    sgCopyVec4( panel_color, l->scene_diffuse());
+    if ( fgGetDouble("/systems/electrical/outputs/instrument-lights") > 1.0 ) {
+        if ( panel_color[0] < 0.7 ) panel_color[0] = 0.7;
+        if ( panel_color[1] < 0.2 ) panel_color[1] = 0.2;
+        if ( panel_color[2] < 0.2 ) panel_color[2] = 0.2;
+    }
+    glColor4fv( panel_color );
 
     glTexCoord2f(_texture.getMinX(), _texture.getMinY()); glVertex2f(-w2, -h2);
     glTexCoord2f(_texture.getMaxX(), _texture.getMinY()); glVertex2f(w2, -h2);
@@ -824,7 +969,7 @@ FGTexturedLayer::draw ()
 ////////////////////////////////////////////////////////////////////////
 
 FGTextLayer::FGTextLayer (int w, int h)
-  : FGInstrumentLayer(w, h), _pointSize(14.0)
+  : FGInstrumentLayer(w, h), _pointSize(14.0), _font_name("default")
 {
   _then.stamp();
   _color[0] = _color[1] = _color[2] = 0.0;
@@ -844,24 +989,44 @@ void
 FGTextLayer::draw ()
 {
   if (test()) {
-    glPushMatrix();
     glColor4fv(_color);
     transform();
-    text_renderer.setFont(guiFntHandle);
+    if ( _font_name == "led" && led_font != 0) {
+	text_renderer.setFont(led_font);
+    } else {
+	text_renderer.setFont(guiFntHandle);
+    }
     text_renderer.setPointSize(_pointSize);
     text_renderer.begin();
     text_renderer.start3f(0, 0, 0);
 
     _now.stamp();
-    if (_now - _then > 100000) {
+    long diff = _now - _then;
+
+    if (diff > 100000 || diff < 0 ) {
+      // ( diff < 0 ) is a sanity check and indicates our time stamp
+      // difference math probably overflowed.  We can handle a max
+      // difference of 35.8 minutes since the returned value is in
+      // usec.  So if the panel is left off longer than that we can
+      // over flow the math with it is turned back on.  This (diff <
+      // 0) catches that situation, get's us out of trouble, and
+      // back on track.
       recalc_value();
       _then = _now;
     }
+
+    // Something is goofy.  The code in this file renders only CCW
+    // polygons, and I have verified that the font code in plib
+    // renders only CCW trianbles.  Yet they come out backwards.
+    // Something around here or in plib is either changing the winding
+    // order or (more likely) pushing a left-handed matrix onto the
+    // stack.  But I can't find it; get out the chainsaw...
+    glFrontFace(GL_CW);
     text_renderer.puts((char *)(_value.c_str()));
+    glFrontFace(GL_CCW);
 
     text_renderer.end();
     glColor4f(1.0, 1.0, 1.0, 1.0);	// FIXME
-    glPopMatrix();
   }
 }
 
@@ -886,6 +1051,13 @@ FGTextLayer::setPointSize (float size)
   _pointSize = size;
 }
 
+void
+FGTextLayer::setFontName(const string &name)
+{
+  _font_name = name;
+}
+
+
 void
 FGTextLayer::setFont(fntFont * font)
 {
@@ -914,15 +1086,16 @@ FGTextLayer::Chunk::Chunk (const string &text, const string &fmt)
   : _type(FGTextLayer::TEXT), _fmt(fmt)
 {
   _text = text;
-  if (_fmt == "") 
+  if (_fmt.empty()) 
     _fmt = "%s";
 }
 
 FGTextLayer::Chunk::Chunk (ChunkType type, const SGPropertyNode * node,
-			   const string &fmt, float mult)
-  : _type(type), _fmt(fmt), _mult(mult)
+			   const string &fmt, float mult, float offs,
+                           bool truncation)
+  : _type(type), _fmt(fmt), _mult(mult), _offs(offs), _trunc(truncation)
 {
-  if (_fmt == "") {
+  if (_fmt.empty()) {
     if (type == TEXT_VALUE)
       _fmt = "%s";
     else
@@ -935,15 +1108,18 @@ const char *
 FGTextLayer::Chunk::getValue () const
 {
   if (test()) {
+    _buf[0] = '\0';
     switch (_type) {
     case TEXT:
       sprintf(_buf, _fmt.c_str(), _text.c_str());
       return _buf;
     case TEXT_VALUE:
-      sprintf(_buf, _fmt.c_str(), _node->getStringValue().c_str());
+      sprintf(_buf, _fmt.c_str(), _node->getStringValue());
       break;
     case DOUBLE_VALUE:
-      sprintf(_buf, _fmt.c_str(), _node->getFloatValue() * _mult);
+      double d = _offs + _node->getFloatValue() * _mult;
+      if (_trunc)  d = (d < 0) ? -floor(-d) : floor(d);
+      sprintf(_buf, _fmt.c_str(), d);
       break;
     }
     return _buf;
@@ -958,31 +1134,28 @@ FGTextLayer::Chunk::getValue () const
 // Implementation of FGSwitchLayer.
 ////////////////////////////////////////////////////////////////////////
 
-FGSwitchLayer::FGSwitchLayer (int w, int h, const SGPropertyNode * node,
-			      FGInstrumentLayer * layer1,
-			      FGInstrumentLayer * layer2)
-  : FGInstrumentLayer(w, h), _node(node), _layer1(layer1), _layer2(layer2)
+FGSwitchLayer::FGSwitchLayer ()
+  : FGGroupLayer()
 {
 }
 
-FGSwitchLayer::~FGSwitchLayer ()
-{
-  delete _layer1;
-  delete _layer2;
-}
-
 void
 FGSwitchLayer::draw ()
 {
   if (test()) {
     transform();
-    if (_node->getBoolValue()) {
-      _layer1->draw();
-    } else {
-      _layer2->draw();
+    int nLayers = _layers.size();
+    for (int i = 0; i < nLayers; i++) {
+      if (_layers[i]->test()) {
+          _layers[i]->draw();
+          return;
+      }
     }
   }
 }
 
 
 // end of panel.cxx
+
+
+
