@@ -59,24 +59,23 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_FCS "$Id: FGFCS.h,v 1.38 2001/12/21 18:20:21 david Exp $"
+#define ID_FCS "$Id: FGFCS.h,v 1.6 2004/01/14 22:09:40 curt Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-COMMENTS, REFERENCES, and NOTES [use "class documentation" below for API docs]
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
+namespace JSBSim {
+
+typedef enum { iDe=0, iDaL, iDaR, iDr, iDsb, iDsp, iDf, NNorm } FcIdx;
+typedef enum { ofRad=0, ofNorm, ofMag , NForms} OutputForm;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 /** Encapsulates the Flight Control System (FCS) functionality.
-    <ul><li>\URL[Source Code]{FGFCS.cpp.html}</li>
-    <li>\URL[Header File]{FGFCS.h.html}</li></ul>
-    This class owns and contains the list of \URL[components]{FGFCSComponent.html}
+    This class owns and contains the list of FGFCSComponents
     that define the control system for this aircraft. The config file for the
     aircraft contains a description of the control path that starts at an input
     or command and ends at an effector, e.g. an aerosurface. The FCS components
@@ -138,7 +137,7 @@ CLASS DOCUMENTATION
     individual components for more information on how they are mechanized.
     
     @author Jon S. Berndt
-    @version $Id: FGFCS.h,v 1.38 2001/12/21 18:20:21 david Exp $
+    @version $Id: FGFCS.h,v 1.6 2004/01/14 22:09:40 curt Exp $
     @see FGFCSComponent
     @see FGConfigFile
     @see FGGain
@@ -170,105 +169,250 @@ public:
   //@{
   /** Gets the aileron command.
       @return aileron command in percent */
-  inline double GetDaCmd(void) { return DaCmd; }
-
+  inline double GetDaCmd(void) const { return DaCmd; }
+  
   /** Gets the elevator command.
       @return elevator command in percent */
-  inline double GetDeCmd(void) { return DeCmd; }
+  inline double GetDeCmd(void) const { return DeCmd; }
 
   /** Gets the rudder command.
       @return rudder command in percent */
-  inline double GetDrCmd(void) { return DrCmd; }
+  inline double GetDrCmd(void) const { return DrCmd; }
 
   /** Gets the flaps command.
       @return flaps command in percent */
-  inline double GetDfCmd(void) { return DfCmd; }
+  inline double GetDfCmd(void) const { return DfCmd; }
 
   /** Gets the speedbrake command.
       @return speedbrake command in percent */
-  inline double GetDsbCmd(void) { return DsbCmd; }
+  inline double GetDsbCmd(void) const { return DsbCmd; }
 
   /** Gets the spoiler command.
       @return spoiler command in percent */
-  inline double GetDspCmd(void) { return DspCmd; }
+  inline double GetDspCmd(void) const { return DspCmd; }
 
   /** Gets the throttle command.
       @param engine engine ID number
       @return throttle command in percent ( 0 - 100) for the given engine */
-  double GetThrottleCmd(int engine);
+  double GetThrottleCmd(int engine) const;
 
   /** Gets the mixture command.
       @param engine engine ID number
       @return mixture command in percent ( 0 - 100) for the given engine */
-  inline double GetMixtureCmd(int engine) { return MixtureCmd[engine]; }
+  inline double GetMixtureCmd(int engine) const { return MixtureCmd[engine]; }
 
   /** Gets the prop pitch command.
       @param engine engine ID number
       @return pitch command in percent ( 0.0 - 1.0) for the given engine */
-  inline double GetPropAdvanceCmd(int engine) { return PropAdvanceCmd[engine]; }
+  inline double GetPropAdvanceCmd(int engine) const { return PropAdvanceCmd[engine]; }
 
   /** Gets the pitch trim command.
       @return pitch trim command in percent */
-  inline double GetPitchTrimCmd(void) { return PTrimCmd; }
+  inline double GetPitchTrimCmd(void) const { return PTrimCmd; }
   
   /** Gets the rudder trim command.
       @return rudder trim command in percent */
-  inline double GetYawTrimCmd(void) { return YTrimCmd; }
+  inline double GetYawTrimCmd(void) const { return YTrimCmd; }
   
   /** Gets the aileron trim command.
       @return aileron trim command in percent */
-  inline double GetRollTrimCmd(void) { return RTrimCmd; }
+  inline double GetRollTrimCmd(void) const { return RTrimCmd; }
   
   /** Get the gear extend/retract command. 0 commands gear up, 1 down.
       defaults to down.
       @return the current value of the gear extend/retract command*/
-  inline double GetGearCmd(void) { return GearCmd; }    
+  inline double GetGearCmd(void) const { return GearCmd; }    
+  //@}
+
+  /// @name AUTOPilot -> FCS effectors command retrieval
+  //@{
+  /** Gets the AUTOPilot aileron command.
+      @return aileron command in radians */
+  inline double GetAPDaCmd(void) const { return AP_DaCmd; }
+  
+  /** Gets the AUTOPilot elevator command.
+      @return elevator command in radians */
+  inline double GetAPDeCmd(void) const { return AP_DeCmd; }
+
+  /** Gets the AUTOPilot rudder command.
+      @return rudder command in radians */
+  inline double GetAPDrCmd(void) const { return AP_DrCmd; }
+  
+  /** Gets the AUTOPilot throttle (all engines) command.
+      @return throttle command in percent */
+  inline double GetAPThrottleCmd(void) const { return AP_ThrottleCmd; }
+  //@}
+
+  /// @name AUTOPilot setpoint retrieval
+  //@{
+  /** Gets the autopilot pitch attitude setpoint
+      @return Pitch attitude setpoint in radians */
+  inline double GetAPAttitudeSetPt(void) const {return APAttitudeSetPt;}
+
+  /** Gets the autopilot altitude setpoint
+      @return Altitude setpoint in feet */
+  inline double GetAPAltitudeSetPt(void) const {return APAltitudeSetPt;}
+
+  /** Gets the autopilot heading setpoint
+      @return Heading setpoint in radians */
+  inline double GetAPHeadingSetPt(void) const {return APHeadingSetPt;}
+
+  /** Gets the autopilot airspeed setpoint
+      @return Airspeed setpoint in fps */
+  inline double GetAPAirspeedSetPt(void) const {return APAirspeedSetPt;}
+  //@}
+
+  /// @name AUTOPilot setpoint setting
+  //@{
+  /// Sets the autopilot pitch attitude setpoint
+  inline void SetAPAttitudeSetPt(double set) {APAttitudeSetPt = set;}
+
+  /// Sets the autopilot altitude setpoint
+  inline void SetAPAltitudeSetPt(double set) {APAltitudeSetPt = set;}
+
+  /// Sets the autopilot heading setpoint
+  inline void SetAPHeadingSetPt(double set) {APHeadingSetPt = set;}
+
+  /// Sets the autopilot airspeed setpoint
+  inline void SetAPAirspeedSetPt(double set) {APAirspeedSetPt = set;}
+  //@}
+
+
+    /// @name AUTOPilot mode setting
+  //@{
+  /** Turns on/off the attitude-seeking autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPAcquireAttitude(bool set) {APAcquireAttitude = set;}
+  
+  /** Turns on/off the altitude-seeking autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPAcquireAltitude(bool set) {APAcquireAltitude = set;}
+  
+  /** Turns on/off the heading-seeking autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPAcquireHeading(bool set) {APAcquireHeading = set;}
+  
+  /** Turns on/off the airspeed-seeking autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPAcquireAirspeed(bool set) {APAcquireAirspeed = set;}
+  
+  /** Turns on/off the attitude-holding autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPAttitudeHold(bool set) {APAttitudeHold = set;}
+
+  /** Turns on/off the altitude-holding autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPAltitudeHold(bool set) {APAltitudeHold = set;}
+
+  /** Turns on/off the heading-holding autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPHeadingHold(bool set) {APHeadingHold = set;}
+
+  /** Turns on/off the airspeed-holding autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPAirspeedHold(bool set) {APAirspeedHold = set;}
+
+  /** Turns on/off the wing-leveler autopilot.
+      @param set true turns the mode on, false turns it off  **/
+  inline void SetAPWingsLevelHold(bool set) {APWingsLevelHold = set;}
+  //@}
+  
+  /// @name AUTOPilot mode retrieval
+  //@{
+  /** Retrieves the on/off mode of the autopilot AcquireAttitude mode
+      @return true if on, false if off */
+  inline bool GetAPAcquireAttitude(void) const {return APAcquireAttitude;}
+
+  /** Retrieves the on/off mode of the autopilot AcquireAltitude mode
+      @return true if on, false if off */
+  inline bool GetAPAcquireAltitude(void) const {return APAcquireAltitude;}
+
+  /** Retrieves the on/off mode of the autopilot AcquireHeading mode
+      @return true if on, false if off */
+  inline bool GetAPAcquireHeading(void) const {return APAcquireHeading;}
+
+  /** Retrieves the on/off mode of the autopilot AcquireAirspeed mode
+      @return true if on, false if off */
+  inline bool GetAPAcquireAirspeed(void) const {return APAcquireAirspeed;}
+
+  /** Retrieves the on/off mode of the autopilot AttitudeHold mode
+      @return true if on, false if off */
+  inline bool GetAPAttitudeHold(void) const {return APAttitudeHold;}
+
+  /** Retrieves the on/off mode of the autopilot AltitudeHold mode
+      @return true if on, false if off */
+  inline bool GetAPAltitudeHold(void) const {return APAltitudeHold;}
+
+  /** Retrieves the on/off mode of the autopilot HeadingHold mode
+      @return true if on, false if off */
+  inline bool GetAPHeadingHold(void) const {return APHeadingHold;}
+
+  /** Retrieves the on/off mode of the autopilot AirspeedHold mode
+      @return true if on, false if off */
+  inline bool GetAPAirspeedHold(void) const {return APAirspeedHold;}
+
+  /** Retrieves the on/off mode of the autopilot WingsLevelHold mode
+      @return true if on, false if off */
+  inline bool GetAPWingsLevelHold(void) const {return APWingsLevelHold;}
   //@}
 
   /// @name Aerosurface position retrieval
   //@{
-  /** Gets the aileron position.
+  /** Gets the left aileron position.
       @return aileron position in radians */
-  inline double GetDaPos(void) { return DaPos; }
+  inline double GetDaLPos( int form = ofRad ) 
+                         const { return DaLPos[form]; }
+
+  /// @name Aerosurface position retrieval
+  //@{
+  /** Gets the right aileron position.
+      @return aileron position in radians */
+  inline double GetDaRPos( int form = ofRad ) 
+                         const { return DaRPos[form]; }
 
   /** Gets the elevator position.
       @return elevator position in radians */
-  inline double GetDePos(void) { return DePos; }
-
+  inline double GetDePos( int form = ofRad ) 
+                         const { return DePos[form]; }
+ 
   /** Gets the rudder position.
       @return rudder position in radians */
-  inline double GetDrPos(void) { return DrPos; }
-
-  /** Gets the flaps position.
-      @return flaps position in radians */
-  inline double GetDfPos(void) { return DfPos; }
+  inline double GetDrPos( int form = ofRad ) 
+                         const { return DrPos[form]; }
 
   /** Gets the speedbrake position.
       @return speedbrake position in radians */
-  inline double GetDsbPos(void) { return DsbPos; }
+  inline double GetDsbPos( int form = ofRad ) 
+                         const { return DsbPos[form]; }
 
   /** Gets the spoiler position.
       @return spoiler position in radians */
-  inline double GetDspPos(void) { return DspPos; }
-
+  inline double GetDspPos( int form = ofRad ) 
+                         const { return DspPos[form]; }
+  
+  /** Gets the flaps position.
+      @return flaps position in radians */
+  inline double GetDfPos( int form = ofRad ) 
+                         const { return DfPos[form]; }
+                         
   /** Gets the throttle position.
       @param engine engine ID number
       @return throttle position for the given engine in percent ( 0 - 100)*/
-  double GetThrottlePos(int engine);
+  double GetThrottlePos(int engine) const;
 
   /** Gets the mixture position.
       @param engine engine ID number
       @return mixture position for the given engine in percent ( 0 - 100)*/
-  inline double GetMixturePos(int engine) { return MixturePos[engine]; }
+  inline double GetMixturePos(int engine) const { return MixturePos[engine]; }
   
   /** Gets the gear position (0 up, 1 down), defaults to down
       @return gear position (0 up, 1 down) */
-  inline double GetGearPos(void) { return GearPos; }    
+  inline double GetGearPos(void) const { return GearPos; }    
 
   /** Gets the prop pitch position.
       @param engine engine ID number
       @return prop pitch position for the given engine in percent ( 0.0-1.0)*/
-  inline double GetPropAdvance(int engine) { return PropAdvance[engine]; }
+  inline double GetPropAdvance(int engine) const { return PropAdvance[engine]; }
   //@}
 
   /** Retrieves the State object pointer.
@@ -279,7 +423,7 @@ public:
   /** Retrieves a components output value
       @param idx the index of the component (the component ID)
       @return output value from the component */
-  double GetComponentOutput(eParam idx);
+  double GetComponentOutput(int idx);
 
   /** Retrieves the component name
       @param idx the index of the component (the component ID)
@@ -296,11 +440,11 @@ public:
   //@{
   /** Sets the aileron command
       @param cmd aileron command in percent*/
-  inline void SetDaCmd(double cmd) { DaCmd = cmd; }
+  inline void SetDaCmd( double cmd ) { DaCmd = cmd; }
 
   /** Sets the elevator command
       @param cmd elevator command in percent*/
-  inline void SetDeCmd(double cmd) { DeCmd = cmd; }
+  inline void SetDeCmd(double cmd ) { DeCmd = cmd; }
 
   /** Sets the rudder command
       @param cmd rudder command in percent*/
@@ -350,32 +494,62 @@ public:
   void SetPropAdvanceCmd(int engine, double cmd);
   //@}
 
+  /// @name AUTOPilot -> FCS effector command setting
+  //@{
+  /** Sets the AUTOPilot aileron command
+      @param cmd AUTOPilot aileron command in radians*/
+  inline void SetAPDaCmd( double cmd ) { AP_DaCmd = cmd; }
+
+  /** Sets the AUTOPilot elevator command
+      @param cmd AUTOPilot elevator command in radians*/
+  inline void SetAPDeCmd(double cmd ) { AP_DeCmd = cmd; }
+
+  /** Sets the AUTOPilot rudder command
+      @param cmd AUTOPilot rudder command in radians*/
+  inline void SetAPDrCmd(double cmd) { AP_DrCmd = cmd; }
+
+  /** Sets the AUTOPilot throttle command
+      @param cmd AUTOPilot throttle command in percent*/
+  inline void SetAPThrottleCmd(double cmd) { AP_ThrottleCmd = cmd; }
+  //@}
+
   /// @name Aerosurface position setting
   //@{
-  /** Sets the aileron position
-      @param cmd aileron position in radians*/
-  inline void SetDaPos(double cmd) { DaPos = cmd; }
+  /** Sets the left aileron position
+      @param cmd left aileron position in radians*/
+  inline void SetDaLPos( int form , double pos ) 
+                                      { DaLPos[form] = pos; }
+
+  /** Sets the right aileron position
+      @param cmd right aileron position in radians*/
+  inline void SetDaRPos( int form , double pos ) 
+                                      { DaRPos[form] = pos; }
 
   /** Sets the elevator position
       @param cmd elevator position in radians*/
-  inline void SetDePos(double cmd) { DePos = cmd; }
+  inline void SetDePos( int form , double pos ) 
+                                      { DePos[form] = pos; }
 
   /** Sets the rudder position
       @param cmd rudder position in radians*/
-  inline void SetDrPos(double cmd) { DrPos = cmd; }
-
-  /** Sets the flaps position
+  inline void SetDrPos( int form , double pos ) 
+                                      { DrPos[form] = pos; }
+ 
+   /** Sets the flaps position
       @param cmd flaps position in radians*/
-  inline void SetDfPos(double cmd) { DfPos = cmd; }
-
+  inline void SetDfPos( int form , double pos ) 
+                                      { DfPos[form] = pos; }
+  
   /** Sets the speedbrake position
       @param cmd speedbrake position in radians*/
-  inline void SetDsbPos(double cmd) { DsbPos = cmd; }
+  inline void SetDsbPos( int form , double pos ) 
+                                      { DsbPos[form] = pos; }
 
   /** Sets the spoiler position
       @param cmd spoiler position in radians*/
-  inline void SetDspPos(double cmd) { DspPos = cmd; }
-
+  inline void SetDspPos( int form , double pos ) 
+                                      { DspPos[form] = pos; }
+ 
   /** Sets the actual throttle setting for the specified engine
       @param engine engine ID number
       @param cmd throttle setting in percent (0 - 100)*/
@@ -397,7 +571,7 @@ public:
   void SetPropAdvance(int engine, double cmd);
   //@}
 
-  /// @name Landing Gear brakes
+    /// @name Landing Gear brakes
   //@{
   /** Sets the left brake group
       @param cmd brake setting in percent (0.0 - 1.0) */
@@ -426,10 +600,18 @@ public:
   bool Load(FGConfigFile* AC_cfg);
 
   void AddThrottle(void);
-
+  
+  FGPropertyManager* GetPropertyManager(void) { return PropertyManager; }
+  
+  void bind(void);
+  void bindModel(void);
+  void unbind(FGPropertyManager *node);
+  
 private:
   double DaCmd, DeCmd, DrCmd, DfCmd, DsbCmd, DspCmd;
-  double DaPos, DePos, DrPos, DfPos, DsbPos, DspPos;
+  double AP_DaCmd, AP_DeCmd, AP_DrCmd, AP_ThrottleCmd;
+  double DePos[NForms], DaLPos[NForms], DaRPos[NForms], DrPos[NForms];  
+  double DfPos[NForms], DsbPos[NForms], DspPos[NForms];
   double PTrimCmd, YTrimCmd, RTrimCmd;
   vector <double> ThrottleCmd;
   vector <double> ThrottlePos;
@@ -440,11 +622,21 @@ private:
   double LeftBrake, RightBrake, CenterBrake; // Brake settings
   double GearCmd,GearPos;
 
-  vector <FGFCSComponent*> Components;
+  enum Mode {mAP, mFCS, mNone} eMode;
+
+  double APAttitudeSetPt, APAltitudeSetPt, APHeadingSetPt, APAirspeedSetPt;
+  bool APAcquireAttitude, APAcquireAltitude, APAcquireHeading, APAcquireAirspeed;
+  bool APAttitudeHold, APAltitudeHold, APHeadingHold, APAirspeedHold, APWingsLevelHold;
+
+  bool DoNormalize;
+  void Normalize(void);
+
+  vector <FGFCSComponent*> FCSComponents;
+  vector <FGFCSComponent*> APComponents;
+  int ToNormalize[NNorm];
   void Debug(int from);
 };
-
-#include "FGState.h"
+}
 
 #endif
 
