@@ -46,7 +46,7 @@ INCLUDES
 #  include STL_IOSTREAM
 #  include STL_ITERATOR
 #else
-#  if defined(sgi) && !defined(__GNUC__)
+#  if defined(sgi) && !defined(__GNUC__) && (_COMPILER_VERSION < 740)
 #    include <iostream.h>
 #  else
 #    include <iostream>
@@ -71,8 +71,11 @@ INCLUDES
 #include "FGOutput.h"
 #include "FGConfigFile.h"
 #include "FGInitialCondition.h"
+#include "FGPropertyManager.h"
 
-static const char *IdSrc = "$Id: FGFDMExec.cpp,v 1.40 2001/12/24 12:54:55 david Exp $";
+namespace JSBSim {
+
+static const char *IdSrc = "$Id: FGFDMExec.cpp,v 1.9 2004/01/19 13:51:05 ehofman Exp $";
 static const char *IdHdr = ID_FDMEXEC;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -80,15 +83,33 @@ GLOBAL DECLARATIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 unsigned int FGFDMExec::FDMctr = 0;
+FGPropertyManager* FGFDMExec::master=0;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+void checkTied ( FGPropertyManager *node )
+{
+  int N = node->nChildren();
+  string name;
+
+  for (int i=0; i<N; i++) {
+    if (node->getChild(i)->nChildren() ) {
+      checkTied( (FGPropertyManager*)node->getChild(i) );
+    } else if ( node->getChild(i)->isTied() ) {
+      name = ((FGPropertyManager*)node->getChild(i))->GetFullyQualifiedName();
+      cerr << name << " is tied" << endl;
+    } 
+  }
+}        
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 // Constructor
 
-FGFDMExec::FGFDMExec(void)
+FGFDMExec::FGFDMExec(FGPropertyManager* root)
 {
+  
   Frame           = 0;
   FirstModel      = 0;
   Error           = 0;
@@ -106,33 +127,56 @@ FGFDMExec::FGFDMExec(void)
   Position        = 0;
   Auxiliary       = 0;
   Output          = 0;
+  IC              = 0;
+  Trim            = 0;
 
   terminate = false;
   frozen = false;
   modelLoaded = false;
+  IsSlave = false;
 
   IdFDM = FDMctr;
   FDMctr++;
 
   try {
     char* num = getenv("JSBSIM_DEBUG");
-    if (!num) debug_lvl = 1;
-    else debug_lvl = atoi(num); // set debug level
+    if (num) debug_lvl = atoi(num); // set debug level
   } catch (...) {               // if error set to 1
     debug_lvl = 1;
   }
 
-  Debug(0);
+  if (root == 0)  master= new FGPropertyManager;
+  else            master = root;
 
-  Allocate();
+  instance = master->GetNode("/fdm/jsbsim",IdFDM,true);
+
+  
+  Debug(0);
+  
+  // this is here to catch errors in binding member functions
+  // to the property tree.
+  try {
+    Allocate();
+  } catch ( string msg ) {
+    cout << "Caught error: " << msg << endl;
+    exit(1);
+  }    
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 FGFDMExec::~FGFDMExec()
 {
-  DeAllocate();
-
+  try {
+    DeAllocate();
+    checkTied( instance );
+  } catch ( string msg ) {
+    cout << "Caught error: " << msg << endl;
+  }    
+  
+  for (unsigned int i=1; i<SlaveFDMList.size(); i++) delete SlaveFDMList[i]->exec;
+  SlaveFDMList.clear();
+  
   Debug(1);
 }
 
@@ -186,7 +230,7 @@ bool FGFDMExec::Allocate(void)
   if (!Aircraft->InitModel())   {
     cerr << fgred << "Aircraft model init failed" << fgdef << endl;
     Error+=128;}
-  if (!Translation->InitModel()){
+  if (!Translation->InitModel()) {
     cerr << fgred << "Translation model init failed" << fgdef << endl;
     Error+=256;}
   if (!Rotation->InitModel())   {
@@ -203,10 +247,13 @@ bool FGFDMExec::Allocate(void)
     Error+=4096;}
 
   if (Error > 0) result = false;
-
+  
+  IC = new FGInitialCondition(this); 
+  
   // Schedule a model. The second arg (the integer) is the pass number. For
   // instance, the atmosphere model gets executed every fifth pass it is called
   // by the executive. Everything else here gets executed each pass.
+  // IC and Trim objects are NOT scheduled.
 
   Schedule(Atmosphere,      1);
   Schedule(FCS,             1);
@@ -229,23 +276,26 @@ bool FGFDMExec::Allocate(void)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-bool FGFDMExec::DeAllocate(void) {
-
-  if ( Atmosphere != 0 )     delete Atmosphere;
-  if ( FCS != 0 )            delete FCS;
-  if ( Propulsion != 0)      delete Propulsion;
-  if ( MassBalance != 0)     delete MassBalance;
-  if ( Aerodynamics != 0)    delete Aerodynamics;
-  if ( Inertial != 0)        delete Inertial;
-  if ( GroundReactions != 0) delete GroundReactions;
-  if ( Aircraft != 0 )       delete Aircraft;
-  if ( Translation != 0 )    delete Translation;
-  if ( Rotation != 0 )       delete Rotation;
-  if ( Position != 0 )       delete Position;
-  if ( Auxiliary != 0 )      delete Auxiliary;
-  if ( Output != 0 )         delete Output;
-  if ( State != 0 )          delete State;
-
+bool FGFDMExec::DeAllocate(void)
+{
+  delete Atmosphere;
+  delete FCS;
+  delete Propulsion;
+  delete MassBalance;
+  delete Aerodynamics;
+  delete Inertial;
+  delete GroundReactions;
+  delete Aircraft;
+  delete Translation;
+  delete Rotation;
+  delete Position;
+  delete Auxiliary;
+  delete Output;
+  delete State;
+  
+  delete IC;
+  delete Trim;
+    
   FirstModel  = 0L;
   Error       = 0;
 
@@ -308,39 +358,72 @@ bool FGFDMExec::Run(void)
 
   Debug(2);
 
-  while (!model_iterator->Run()) {
+  for (unsigned int i=1; i<SlaveFDMList.size(); i++) {
+//    SlaveFDMList[i]->exec->State->Initialize(); // Transfer state to the slave FDM
+//    SlaveFDMList[i]->exec->Run();
+  }
+
+  while (model_iterator != 0L) {
+    model_iterator->Run();
     model_iterator = model_iterator->NextModel;
-    if (model_iterator == 0L) break;
   }
 
   frame = Frame++;
   State->IncrTime();
-
   return true;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-bool FGFDMExec::RunIC(FGInitialCondition *fgic)
+bool FGFDMExec::RunIC(void)
 {
   State->Suspend();
-  State->Initialize(fgic);
+  State->Initialize(IC);
   Run();
   State->Resume();
+
   return true;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-bool FGFDMExec::LoadModel(string APath, string EPath, string model)
+vector <string> FGFDMExec::EnumerateFDMs(void)
+{
+  vector <string> FDMList;
+
+  FDMList.push_back(Aircraft->GetAircraftName());
+
+  for (unsigned int i=1; i<SlaveFDMList.size(); i++) {
+    FDMList.push_back(SlaveFDMList[i]->exec->GetAircraft()->GetAircraftName());
+  }
+
+  return FDMList;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+bool FGFDMExec::LoadModel(string AircraftPath, string EnginePath, string model)
+{
+  FGFDMExec::AircraftPath = AircraftPath;
+  FGFDMExec::EnginePath = EnginePath;
+
+  return LoadModel(model);
+}  
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+bool FGFDMExec::LoadModel(string model)
 {
   bool result = true;
   string token;
   string aircraftCfgFileName;
 
-  AircraftPath = APath;
-  EnginePath   = EPath;
-
+  if( AircraftPath.empty() || EnginePath.empty() ) {
+    cerr << "Error: attempted to load aircraft with undefined ";
+    cerr << "aircraft and engine paths" << endl;
+    return false;
+  }
+    
 # ifndef macintosh
   aircraftCfgFileName = AircraftPath + "/" + model + "/" + model + ".xml";
 # else
@@ -349,7 +432,9 @@ bool FGFDMExec::LoadModel(string APath, string EPath, string model)
 
   FGConfigFile AC_cfg(aircraftCfgFileName);
   if (!AC_cfg.IsOpen()) return false;
-
+  
+  modelName = model;
+  
   if (modelLoaded) {
     DeAllocate();
     Allocate();
@@ -362,6 +447,10 @@ bool FGFDMExec::LoadModel(string APath, string EPath, string model)
     if (token == "METRICS") {
       if (debug_lvl > 0) cout << fgcyan << "\n  Reading Metrics" << fgdef << endl;
       if (!ReadMetrics(&AC_cfg)) result = false;
+    } else if (token == "SLAVE") {
+      if (debug_lvl > 0) cout << fgcyan << "\n  Reading Slave flight vehicle: " << fgdef
+                                        << AC_cfg.GetValue("NAME") << endl;
+      if (!ReadSlave(&AC_cfg)) result = false;
     } else if (token == "AERODYNAMICS") {
       if (debug_lvl > 0) cout << fgcyan << "\n  Reading Aerodynamics" << fgdef << endl;
       if (!ReadAerodynamics(&AC_cfg)) result = false;
@@ -374,6 +463,9 @@ bool FGFDMExec::LoadModel(string APath, string EPath, string model)
     } else if (token == "FLIGHT_CONTROL") {
       if (debug_lvl > 0) cout << fgcyan << "\n  Reading Flight Control" << fgdef << endl;
       if (!ReadFlightControls(&AC_cfg)) result = false;
+    } else if (token == "AUTOPILOT") {
+      if (debug_lvl > 0) cout << fgcyan << "\n  Reading Autopilot" << fgdef << endl;
+      if (!ReadFlightControls(&AC_cfg)) result = false;
     } else if (token == "OUTPUT") {
       if (debug_lvl > 0) cout << fgcyan << "\n  Reading Output directives" << fgdef << endl;
       if (!ReadOutput(&AC_cfg)) result = false;
@@ -399,7 +491,7 @@ bool FGFDMExec::ReadPrologue(FGConfigFile* AC_cfg)
   string token = AC_cfg->GetValue();
   string scratch;
   string AircraftName;
-  
+
   AircraftName = AC_cfg->GetValue("NAME");
   Aircraft->SetAircraftName(AircraftName);
 
@@ -408,10 +500,11 @@ bool FGFDMExec::ReadPrologue(FGConfigFile* AC_cfg)
   scratch = AC_cfg->GetValue("VERSION").c_str();
 
   CFGVersion = AC_cfg->GetValue("VERSION");
+  Release    = AC_cfg->GetValue("RELEASE");
 
   if (debug_lvl > 0)
     cout << "                            Version: " << highint << CFGVersion
-                                                             << normint << endl;
+                                                    << normint << endl;
   if (CFGVersion != needed_cfg_version) {
     cerr << endl << fgred << "YOU HAVE AN INCOMPATIBLE CFG FILE FOR THIS AIRCRAFT."
             " RESULTS WILL BE UNPREDICTABLE !!" << endl;
@@ -419,6 +512,78 @@ bool FGFDMExec::ReadPrologue(FGConfigFile* AC_cfg)
     cerr << "         You have version: " << CFGVersion << endl << fgdef << endl;
     return false;
   }
+  
+  if (Release == "ALPHA") {
+    system("banner ALPHA");
+    cout << endl << endl
+         << highint << "This aircraft model is an " << fgred << Release
+         << reset << highint << " release!!!" << endl << endl << reset
+         << "This aircraft model may not even properly load, and probably"
+         << " will not fly as expected." << endl << endl
+         << fgred << highint << "Use this model for development purposes ONLY!!!"
+         << normint << reset << endl << endl;
+  } else if (Release == "BETA") {
+    system("banner BETA");
+    cout << endl << endl
+         << highint << "This aircraft model is a " << fgred << Release
+         << reset << highint << " release!!!" << endl << endl << reset
+         << "This aircraft model probably will not fly as expected." << endl << endl
+         << fgblue << highint << "Use this model for development purposes ONLY!!!"
+         << normint << reset << endl << endl;
+  }
+
+  return true;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+bool FGFDMExec::ReadSlave(FGConfigFile* AC_cfg)
+{
+  // Add a new slaveData object to the slave FDM list
+  // Populate that slaveData element with a new FDMExec object
+  // Set the IsSlave flag for that FDMExec object
+  // Get the aircraft name
+  // set debug level to print out no additional data for slave objects
+  // Load the model given the aircraft name
+  // reset debug level to prior setting
+
+  int saved_debug_lvl = debug_lvl;
+  string token;
+
+  SlaveFDMList.push_back(new slaveData);
+  SlaveFDMList.back()->exec = new FGFDMExec();
+  SlaveFDMList.back()->exec->SetSlave();
+
+  string AircraftName = AC_cfg->GetValue("FILE");
+
+  debug_lvl = 0;                 // turn off debug output for slave vehicle
+  
+  SlaveFDMList.back()->exec->SetAircraftPath( AircraftPath );
+  SlaveFDMList.back()->exec->SetEnginePath( EnginePath );
+  SlaveFDMList.back()->exec->LoadModel(AircraftName);
+  debug_lvl = saved_debug_lvl;   // turn debug output back on for master vehicle
+
+  AC_cfg->GetNextConfigLine();
+  while ((token = AC_cfg->GetValue()) != string("/SLAVE")) {
+    *AC_cfg >> token;
+    if      (token == "XLOC")  { *AC_cfg >> SlaveFDMList.back()->x;    }
+    else if (token == "YLOC")  { *AC_cfg >> SlaveFDMList.back()->y;    }
+    else if (token == "ZLOC")  { *AC_cfg >> SlaveFDMList.back()->z;    }
+    else if (token == "PITCH") { *AC_cfg >> SlaveFDMList.back()->pitch;}
+    else if (token == "YAW")   { *AC_cfg >> SlaveFDMList.back()->yaw;  }
+    else if (token == "ROLL")  { *AC_cfg >> SlaveFDMList.back()->roll;  }
+    else cerr << "Unknown identifier: " << token << " in slave vehicle definition" << endl;
+  }
+
+  if (debug_lvl > 0)  {
+    cout << "      X = " << SlaveFDMList.back()->x << endl;
+    cout << "      Y = " << SlaveFDMList.back()->y << endl;
+    cout << "      Z = " << SlaveFDMList.back()->z << endl;
+    cout << "      Pitch = " << SlaveFDMList.back()->pitch << endl;
+    cout << "      Yaw = " << SlaveFDMList.back()->yaw << endl;
+    cout << "      Roll = " << SlaveFDMList.back()->roll << endl;
+  }
+
   return true;
 }
 
@@ -488,6 +653,20 @@ bool FGFDMExec::ReadOutput(FGConfigFile* AC_cfg)
   return true;
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+FGPropertyManager* FGFDMExec::GetPropertyManager(void) { 
+  return instance;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+FGTrim* FGFDMExec::GetTrim(void) { 
+  delete Trim;
+  Trim = new FGTrim(this,tNone);
+  return Trim;
+}
+  
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 //    The bitmasked value choices are as follows:
 //    unset: In this case (the default) JSBSim would only print
@@ -511,7 +690,7 @@ void FGFDMExec::Debug(int from)
 {
   if (debug_lvl <= 0) return;
 
-  if (debug_lvl & 1) { // Standard console startup message output
+  if (debug_lvl & 1 && IdFDM == 0) { // Standard console startup message output
     if (from == 0) { // Constructor
       cout << "\n\n     " << highint << underon << "JSBSim Flight Dynamics Model v"
                                      << JSBSim_version << underoff << normint << endl;
@@ -542,4 +721,5 @@ void FGFDMExec::Debug(int from)
     }
   }
 }
+}
 
