@@ -36,10 +36,22 @@ INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #include "FGGroundReactions.h"
+#include "FGPropertyManager.h"
 
-static const char *IdSrc = "$Id: FGGroundReactions.cpp,v 1.25 2002/02/01 01:11:13 david Exp $";
+namespace JSBSim {
+
+static const char *IdSrc = "$Id: FGGroundReactions.cpp,v 1.4 2004/01/14 22:09:40 curt Exp $";
 static const char *IdHdr = ID_GROUNDREACTIONS;
 
+#if defined (__APPLE__)
+/* Not all systems have the gcvt function */
+inline char* gcvt (double value, int ndigits, char *buf) {
+    /* note that this is not exactly what gcvt is supposed to do! */
+    snprintf (buf, ndigits+1, "%f", value);
+    return buf;
+}
+#endif
+
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -48,6 +60,8 @@ CLASS IMPLEMENTATION
 FGGroundReactions::FGGroundReactions(FGFDMExec* fgex) : FGModel(fgex)
 {
   Name = "FGGroundReactions";
+  
+  bind();
 
   Debug(0);
 }
@@ -56,6 +70,8 @@ FGGroundReactions::FGGroundReactions(FGFDMExec* fgex) : FGModel(fgex)
 
 FGGroundReactions::~FGGroundReactions(void)
 {
+  unbind();
+
   Debug(1);
 }
 
@@ -120,6 +136,8 @@ string FGGroundReactions::GetGroundReactionStrings(void)
     GroundReactionStrings += (lGear[i].GetName() + "_strokeVel, ");
     GroundReactionStrings += (lGear[i].GetName() + "_CompressForce, ");
     GroundReactionStrings += (lGear[i].GetName() + "_WhlSideForce, ");
+    GroundReactionStrings += (lGear[i].GetName() + "_WhlVelVecX, ");
+    GroundReactionStrings += (lGear[i].GetName() + "_WhlVelVecY, ");
     GroundReactionStrings += (lGear[i].GetName() + "_WhlRollForce, ");
     GroundReactionStrings += (lGear[i].GetName() + "_BodyXForce, ");
     GroundReactionStrings += (lGear[i].GetName() + "_BodyYForce, ");
@@ -128,6 +146,13 @@ string FGGroundReactions::GetGroundReactionStrings(void)
     firstime = false;
   }
 
+  GroundReactionStrings += ", TotalGearForce_X, ";
+  GroundReactionStrings += "TotalGearForce_Y, ";
+  GroundReactionStrings += "TotalGearForce_Z, ";
+  GroundReactionStrings += "TotalGearMoment_L, ";
+  GroundReactionStrings += "TotalGearMoment_M, ";
+  GroundReactionStrings += "TotalGearMoment_N";
+
   return GroundReactionStrings;
 }
 
@@ -146,6 +171,8 @@ string FGGroundReactions::GetGroundReactionValues(void)
     GroundReactionValues += (string(gcvt(lGear[i].GetCompLen(),    5, buff)) + ", ");
     GroundReactionValues += (string(gcvt(lGear[i].GetCompVel(),    6, buff)) + ", ");
     GroundReactionValues += (string(gcvt(lGear[i].GetCompForce(), 10, buff)) + ", ");
+    GroundReactionValues += (string(gcvt(lGear[i].GetWheelVel(eX), 6, buff)) + ", ");
+    GroundReactionValues += (string(gcvt(lGear[i].GetWheelVel(eY), 6, buff)) + ", ");
     GroundReactionValues += (string(gcvt(lGear[i].GetWheelSideForce(), 6, buff)) + ", ");
     GroundReactionValues += (string(gcvt(lGear[i].GetWheelRollForce(), 6, buff)) + ", ");
     GroundReactionValues += (string(gcvt(lGear[i].GetBodyXForce(), 6, buff)) + ", ");
@@ -155,9 +182,50 @@ string FGGroundReactions::GetGroundReactionValues(void)
     firstime = false;
   }
 
+  GroundReactionValues += (", " + string(gcvt(vForces(eX), 6, buff)) + ", ");
+  GroundReactionValues += (string(gcvt(vForces(eY), 6, buff)) + ", ");
+  GroundReactionValues += (string(gcvt(vForces(eZ), 6, buff)) + ", ");
+  GroundReactionValues += (string(gcvt(vMoments(eX), 6, buff)) + ", ");
+  GroundReactionValues += (string(gcvt(vMoments(eY), 6, buff)) + ", ");
+  GroundReactionValues += (string(gcvt(vMoments(eZ), 6, buff)));
+
   return GroundReactionValues;
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGGroundReactions::bind(void)
+{ 
+  typedef double (FGGroundReactions::*PMF)(int) const;
+  PropertyManager->Tie("gear/num-units", this,
+                       &FGGroundReactions::GetNumGearUnits);  
+  PropertyManager->Tie("moments/l-gear-lbsft", this,1,
+                       (PMF)&FGGroundReactions::GetMoments);
+  PropertyManager->Tie("moments/m-gear-lbsft", this,2,
+                       (PMF)&FGGroundReactions::GetMoments);
+  PropertyManager->Tie("moments/n-gear-lbsft", this,3,
+                       (PMF)&FGGroundReactions::GetMoments);
+  PropertyManager->Tie("forces/fbx-gear-lbs", this,1,
+                       (PMF)&FGGroundReactions::GetForces);
+  PropertyManager->Tie("forces/fby-gear-lbs", this,2,
+                       (PMF)&FGGroundReactions::GetForces);
+  PropertyManager->Tie("forces/fbz-gear-lbs", this,3,
+                       (PMF)&FGGroundReactions::GetForces);
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGGroundReactions::unbind(void)
+{
+  PropertyManager->Untie("gear/num-units");
+  PropertyManager->Untie("moments/l-gear-lbsft");
+  PropertyManager->Untie("moments/m-gear-lbsft");
+  PropertyManager->Untie("moments/n-gear-lbsft");
+  PropertyManager->Untie("forces/fbx-gear-lbs");
+  PropertyManager->Untie("forces/fby-gear-lbs");
+  PropertyManager->Untie("forces/fbz-gear-lbs");
+}
+
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 //    The bitmasked value choices are as follows:
 //    unset: In this case (the default) JSBSim would only print
@@ -203,4 +271,4 @@ void FGGroundReactions::Debug(int from)
     }
   }
 }
-
+}
