@@ -43,8 +43,11 @@ INCLUDES
 #endif
 
 #include "FGNozzle.h"
+#include "FGAtmosphere.h"
 
-static const char *IdSrc = "$Id: FGNozzle.cpp,v 1.23 2001/12/24 12:54:55 david Exp $";
+namespace JSBSim {
+
+static const char *IdSrc = "$Id: FGNozzle.cpp,v 1.5 2004/03/14 14:57:07 ehofman Exp $";
 static const char *IdHdr = ID_NOZZLE;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -68,6 +71,7 @@ FGNozzle::FGNozzle(FGFDMExec* FDMExec, FGConfigFile* Nzl_cfg) : FGThruster(FDMEx
   }
 
   Thrust = 0;
+  ReverserAngle = 0.0;
   Type = ttNozzle;
   Area2 = (Diameter*Diameter/4.0)*M_PI;
   AreaT = Area2/ExpR;
@@ -88,7 +92,7 @@ double FGNozzle::Calculate(double CfPc)
 {
   double pAtm = fdmex->GetAtmosphere()->GetPressure();
   Thrust = max((double)0.0, (CfPc * AreaT + (PE - pAtm)*Area2) * nzlEff);
-  vFn(1) = Thrust;
+  vFn(1) = Thrust * cos(ReverserAngle);
 
   return Thrust;
 }
@@ -149,4 +153,4 @@ void FGNozzle::Debug(int from)
     }
   }
 }
-
+}
