@@ -28,7 +28,7 @@
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
 This is the place where you create output routines to dump data for perusal
-later. 
+later.
 
 HISTORY
 --------------------------------------------------------------------------------
@@ -51,8 +51,11 @@ INCLUDES
 #include "FGRotation.h"
 #include "FGPosition.h"
 #include "FGAuxiliary.h"
+#include "FGInertial.h"
 
-static const char *IdSrc = "$Id: FGOutput.cpp,v 1.42 2002/02/06 03:07:12 david Exp $";
+namespace JSBSim {
+
+static const char *IdSrc = "$Id: FGOutput.cpp,v 1.7 2004/03/15 09:24:57 ehofman Exp $";
 static const char *IdHdr = ID_OUTPUT;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -65,10 +68,11 @@ FGOutput::FGOutput(FGFDMExec* fdmex) : FGModel(fdmex)
   sFirstPass = dFirstPass = true;
   socket = 0;
   Type = otNone;
-  Filename = "JSBSim.out";
+  Filename = "";
   SubSystems = 0;
   enabled = true;
-  
+  outputInFileName = "";
+
   Debug(0);
 }
 
@@ -77,6 +81,8 @@ FGOutput::FGOutput(FGFDMExec* fdmex) : FGModel(fdmex)
 FGOutput::~FGOutput()
 {
   if (socket) delete socket;
+  for (unsigned int i=0; i<OutputProperties.size(); i++) delete OutputProperties[i];
+
   Debug(1);
 }
 
@@ -98,8 +104,9 @@ bool FGOutput::Run(void)
       } else {
         // Not a valid type of output
       }
-
+    return false;
     } else {
+    return true;
     }
   }
   return false;
@@ -117,7 +124,7 @@ void FGOutput::SetType(string type)
     Type = otSocket;
   } else if (type == "TERMINAL") {
     Type = otTerminal;
-  } else if (type != string("NONE")){
+  } else if (type != string("NONE")) {
     Type = otUnknown;
     cerr << "Unknown type of output specified in config file" << endl;
   }
@@ -128,6 +135,7 @@ void FGOutput::SetType(string type)
 void FGOutput::DelimitedOutput(string fname)
 {
   streambuf* buffer;
+  string scratch = "";
 
   if (fname == "COUT" || fname == "cout") {
     buffer = cout.rdbuf();
@@ -148,9 +156,12 @@ void FGOutput::DelimitedOutput(string fname)
       outstream << "Aileron Cmd, ";
       outstream << "Elevator Cmd, ";
       outstream << "Rudder Cmd, ";
-      outstream << "Aileron Pos, ";
+      outstream << "Flap Cmd, ";
+      outstream << "Left Aileron Pos, ";
+      outstream << "Right Aileron Pos, ";
       outstream << "Elevator Pos, ";
-      outstream << "Rudder Pos";
+      outstream << "Rudder Pos, ";
+      outstream << "Flap Pos";
     }
     if (SubSystems & ssRates) {
       outstream << ", ";
@@ -169,7 +180,10 @@ void FGOutput::DelimitedOutput(string fname)
       outstream << ", ";
       outstream << "Drag, Side, Lift, ";
       outstream << "L/D, ";
-      outstream << "Xforce, Yforce, Zforce";
+      outstream << "Xforce, Yforce, Zforce, ";
+      outstream << "xGravity, yGravity, zGravity, ";
+      outstream << "xCoriolis, yCoriolis, zCoriolis, ";
+      outstream << "xCentrifugal, yCentrifugal, zCentrifugal";
     }
     if (SubSystems & ssMoments) {
       outstream << ", ";
@@ -183,9 +197,14 @@ void FGOutput::DelimitedOutput(string fname)
     if (SubSystems & ssMassProps) {
       outstream << ", ";
       outstream << "Ixx, ";
-      outstream << "Iyy, ";
-      outstream << "Izz, ";
+      outstream << "Ixy, ";
       outstream << "Ixz, ";
+      outstream << "Iyx, ";
+      outstream << "Iyy, ";
+      outstream << "Iyz, ";
+      outstream << "Izx, ";
+      outstream << "Izy, ";
+      outstream << "Izz, ";
       outstream << "Mass, ";
       outstream << "Xcg, Ycg, Zcg";
     }
@@ -201,8 +220,12 @@ void FGOutput::DelimitedOutput(string fname)
       outstream << "Runway Radius";
     }
     if (SubSystems & ssCoefficients) {
-      outstream << ", ";
-      outstream << Aerodynamics->GetCoefficientStrings();
+      scratch = Aerodynamics->GetCoefficientStrings();
+      if (scratch.length() != 0) outstream << ", " << scratch;
+    }
+    if (SubSystems & ssFCS) {
+      scratch = FCS->GetComponentStrings();
+      if (scratch.length() != 0) outstream << ", " << scratch;
     }
     if (SubSystems & ssGroundReactions) {
       outstream << ", ";
@@ -212,6 +235,11 @@ void FGOutput::DelimitedOutput(string fname)
       outstream << ", ";
       outstream << Propulsion->GetPropulsionStrings();
     }
+    if (OutputProperties.size() > 0) {
+      for (unsigned int i=0;i<OutputProperties.size();i++) {
+        outstream << ", " << OutputProperties[i]->GetName();
+      }
+    }
 
     outstream << endl;
     dFirstPass = false;
@@ -225,9 +253,12 @@ void FGOutput::DelimitedOutput(string fname)
     outstream << FCS->GetDaCmd() << ", ";
     outstream << FCS->GetDeCmd() << ", ";
     outstream << FCS->GetDrCmd() << ", ";
-    outstream << FCS->GetDaPos() << ", ";
+    outstream << FCS->GetDfCmd() << ", ";
+    outstream << FCS->GetDaLPos() << ", ";
+    outstream << FCS->GetDaRPos() << ", ";
     outstream << FCS->GetDePos() << ", ";
-    outstream << FCS->GetDrPos();
+    outstream << FCS->GetDrPos() << ", ";
+    outstream << FCS->GetDfPos();
   }
   if (SubSystems & ssRates) {
     outstream << ", ";
@@ -239,14 +270,17 @@ void FGOutput::DelimitedOutput(string fname)
     outstream << Translation->Getqbar() << ", ";
     outstream << Translation->GetVt() << ", ";
     outstream << Translation->GetUVW() << ", ";
-    outstream << Translation->GetvAeroUVW() << ", ";
+    outstream << Translation->GetAeroUVW() << ", ";
     outstream << Position->GetVel();
   }
   if (SubSystems & ssForces) {
     outstream << ", ";
     outstream << Aerodynamics->GetvFs() << ", ";
     outstream << Aerodynamics->GetLoD() << ", ";
-    outstream << Aircraft->GetForces();
+    outstream << Aircraft->GetForces() << ", ";
+    outstream << Inertial->GetGravity() << ", ";
+    outstream << Inertial->GetCoriolis() << ", ";
+    outstream << Inertial->GetCentrifugal();
   }
   if (SubSystems & ssMoments) {
     outstream << ", ";
@@ -259,10 +293,7 @@ void FGOutput::DelimitedOutput(string fname)
   }
   if (SubSystems & ssMassProps) {
     outstream << ", ";
-    outstream << MassBalance->GetIxx() << ", ";
-    outstream << MassBalance->GetIyy() << ", ";
-    outstream << MassBalance->GetIzz() << ", ";
-    outstream << MassBalance->GetIxz() << ", ";
+    outstream << MassBalance->GetJ() << ", ";
     outstream << MassBalance->GetMass() << ", ";
     outstream << MassBalance->GetXYZcg();
   }
@@ -278,8 +309,12 @@ void FGOutput::DelimitedOutput(string fname)
     outstream << Position->GetRunwayRadius();
   }
   if (SubSystems & ssCoefficients) {
-    outstream << ", ";
-    outstream << Aerodynamics->GetCoefficientValues();
+    scratch = Aerodynamics->GetCoefficientValues();
+    if (scratch.length() != 0) outstream << ", " << scratch;
+  }
+  if (SubSystems & ssFCS) {
+    scratch = FCS->GetComponentValues();
+    if (scratch.length() != 0) outstream << ", " << scratch;
   }
   if (SubSystems & ssGroundReactions) {
     outstream << ", ";
@@ -290,6 +325,10 @@ void FGOutput::DelimitedOutput(string fname)
     outstream << Propulsion->GetPropulsionValues();
   }
 
+  for (unsigned int i=0;i<OutputProperties.size();i++) {
+    outstream << ", " << OutputProperties[i]->getDoubleValue();
+  }
+
   outstream << endl;
   outstream.flush();
 }
@@ -342,7 +381,8 @@ void FGOutput::SocketOutput(void)
     socket->Append("M");
     socket->Append("N");
     socket->Append("Throttle Position");
-    socket->Append("Aileron Position");
+    socket->Append("Left Aileron Position");
+    socket->Append("Right Aileron Position");
     socket->Append("Elevator Position");
     socket->Append("Rudder Position");
     sFirstPass = false;
@@ -360,9 +400,9 @@ void FGOutput::SocketOutput(void)
   socket->Append(Translation->GetUVW(eU));
   socket->Append(Translation->GetUVW(eV));
   socket->Append(Translation->GetUVW(eW));
-  socket->Append(Translation->GetvAeroUVW(eU));
-  socket->Append(Translation->GetvAeroUVW(eV));
-  socket->Append(Translation->GetvAeroUVW(eW));
+  socket->Append(Translation->GetAeroUVW(eU));
+  socket->Append(Translation->GetAeroUVW(eV));
+  socket->Append(Translation->GetAeroUVW(eW));
   socket->Append(Position->GetVn());
   socket->Append(Position->GetVe());
   socket->Append(Position->GetVd());
@@ -386,7 +426,8 @@ void FGOutput::SocketOutput(void)
   socket->Append(Aircraft->GetMoments(eM));
   socket->Append(Aircraft->GetMoments(eN));
   socket->Append(FCS->GetThrottlePos(0));
-  socket->Append(FCS->GetDaPos());
+  socket->Append(FCS->GetDaLPos());
+  socket->Append(FCS->GetDaRPos());
   socket->Append(FCS->GetDePos());
   socket->Append(FCS->GetDrPos());
   socket->Send();
@@ -410,11 +451,20 @@ void FGOutput::SocketStatusOutput(string out_str)
 
 bool FGOutput::Load(FGConfigFile* AC_cfg)
 {
-  string token, parameter;
+  string token="", parameter="", separator="";
+  string name="", fname="";
   int OutRate = 0;
+  FGConfigFile* Output_cfg;
+  string property;
 
-  token = AC_cfg->GetValue("NAME");
-  Output->SetFilename(token);
+# ifndef macintosh
+    separator = "/";
+# else
+    separator = ";";
+# endif
+
+  name = AC_cfg->GetValue("NAME");
+  fname = AC_cfg->GetValue("FILE");
   token = AC_cfg->GetValue("TYPE");
   Output->SetType(token);
 
@@ -423,69 +473,90 @@ bool FGOutput::Load(FGConfigFile* AC_cfg)
     socket = new FGfdmSocket("localhost",1138);
   }
 #endif
-  
-  AC_cfg->GetNextConfigLine();
 
-  while ((token = AC_cfg->GetValue()) != string("/OUTPUT")) {
-    *AC_cfg >> parameter;
-    if (parameter == "RATE_IN_HZ") *AC_cfg >> OutRate;
+  if (!fname.empty()) {
+    outputInFileName = FDMExec->GetAircraftPath() + separator
+                        + FDMExec->GetModelName() + separator + fname + ".xml";
+    Output_cfg = new FGConfigFile(outputInFileName);
+    if (!Output_cfg->IsOpen()) {
+      cerr << "Could not open file: " << outputInFileName << endl;
+      return false;
+    }
+  } else {
+    Output_cfg = AC_cfg;
+  }
+  Output->SetFilename(name);
+
+  while ((token = Output_cfg->GetValue()) != string("/OUTPUT")) {
+    *Output_cfg >> parameter;
+    if (parameter == "RATE_IN_HZ") {
+      *Output_cfg >> OutRate;
+    }
     if (parameter == "SIMULATION") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssSimulation;
     }
     if (parameter == "AEROSURFACES") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssAerosurfaces;
     }
     if (parameter == "RATES") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssRates;
     }
     if (parameter == "VELOCITIES") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssVelocities;
     }
     if (parameter == "FORCES") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssForces;
     }
     if (parameter == "MOMENTS") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssMoments;
     }
     if (parameter == "ATMOSPHERE") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssAtmosphere;
     }
     if (parameter == "MASSPROPS") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssMassProps;
     }
     if (parameter == "POSITION") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssPosition;
     }
     if (parameter == "COEFFICIENTS") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssCoefficients;
     }
     if (parameter == "GROUND_REACTIONS") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssGroundReactions;
     }
     if (parameter == "FCS") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssFCS;
     }
     if (parameter == "PROPULSION") {
-      *AC_cfg >> parameter;
+      *Output_cfg >> parameter;
       if (parameter == "ON") SubSystems += ssPropulsion;
     }
+    if (parameter == "PROPERTY") {
+      *Output_cfg >> property;
+      OutputProperties.push_back(PropertyManager->GetNode(property));
+    }
+
+    if (parameter == "EOF") break;
   }
 
   OutRate = OutRate>120?120:(OutRate<0?0:OutRate);
   rate = (int)(0.5 + 1.0/(State->Getdt()*OutRate));
 
+  Debug(2);
+
   return true;
 }
 
@@ -510,12 +581,48 @@ bool FGOutput::Load(FGConfigFile* AC_cfg)
 
 void FGOutput::Debug(int from)
 {
+  string scratch="";
+
   if (debug_lvl <= 0) return;
 
   if (debug_lvl & 1) { // Standard console startup message output
     if (from == 0) { // Constructor
 
     }
+    if (from == 2) {
+      if (outputInFileName.empty())
+        cout << "  " << "Output parameters read inline" << endl;
+      else
+        cout << "    Output parameters read from file: " << outputInFileName << endl;
+
+      if (Filename == "cout" || Filename == "COUT") {
+        scratch = "    Log output goes to screen console";
+      } else if (!Filename.empty()) {
+        scratch = "    Log output goes to file: " + Filename;
+      }
+      switch (Type) {
+      case otCSV:
+        cout << scratch << " in CSV format output at rate " << 120/rate << " Hz" << endl;
+        break;
+      case otNone:
+        cout << "  No log output" << endl;
+        break;
+      }
+
+      if (SubSystems & ssSimulation)      cout << "    Simulation parameters logged" << endl;
+      if (SubSystems & ssAerosurfaces)    cout << "    Aerosurface parameters logged" << endl;
+      if (SubSystems & ssRates)           cout << "    Rate parameters logged" << endl;
+      if (SubSystems & ssVelocities)      cout << "    Velocity parameters logged" << endl;
+      if (SubSystems & ssForces)          cout << "    Force parameters logged" << endl;
+      if (SubSystems & ssMoments)         cout << "    Moments parameters logged" << endl;
+      if (SubSystems & ssAtmosphere)      cout << "    Atmosphere parameters logged" << endl;
+      if (SubSystems & ssMassProps)       cout << "    Mass parameters logged" << endl;
+      if (SubSystems & ssCoefficients)    cout << "    Coefficient parameters logged" << endl;
+      if (SubSystems & ssPosition)        cout << "    Position parameters logged" << endl;
+      if (SubSystems & ssGroundReactions) cout << "    Ground parameters logged" << endl;
+      if (SubSystems & ssFCS)             cout << "    FCS parameters logged" << endl;
+      if (SubSystems & ssPropulsion)      cout << "    Propulsion parameters logged" << endl;
+    }
   }
   if (debug_lvl & 2 ) { // Instantiation/Destruction notification
     if (from == 0) cout << "Instantiated: FGOutput" << endl;
@@ -534,4 +641,4 @@ void FGOutput::Debug(int from)
     }
   }
 }
-
+}
