@@ -42,7 +42,9 @@ INCLUDES
 #include "FGPiston.h"
 #include "FGPropulsion.h"
 
-static const char *IdSrc = "$Id: FGPiston.cpp,v 1.30 2002/02/13 13:35:10 david Exp $";
+namespace JSBSim {
+
+static const char *IdSrc = "$Id: FGPiston.cpp,v 1.7 2004/02/05 13:47:48 ehofman Exp $";
 static const char *IdHdr = ID_PISTON;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -50,7 +52,6 @@ CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 FGPiston::FGPiston(FGFDMExec* exec, FGConfigFile* Eng_cfg) : FGEngine(exec),
-  CONVERT_CUBIC_INCHES_TO_METERS_CUBED(1.638706e-5),
   R_air(287.3),
   rho_fuel(800),                 // estimate
   calorific_value_fuel(47.3e6),
@@ -59,33 +60,21 @@ FGPiston::FGPiston(FGFDMExec* exec, FGConfigFile* Eng_cfg) : FGEngine(exec),
 {
   string token;
 
-  MinManifoldPressure_inHg = 6.5;
-  MaxManifoldPressure_inHg = 28.5;
-  Displacement = 360;
-  MaxHP = 200;
-  Cycles = 2;
-  IdleRPM = 600;
-
-  Name = Eng_cfg->GetValue("NAME");
-  Eng_cfg->GetNextConfigLine();
-  while (Eng_cfg->GetValue() != string("/FG_PISTON")) {
-    *Eng_cfg >> token;
-    if      (token == "MINMP") *Eng_cfg >> MinManifoldPressure_inHg;
-    else if (token == "MAXMP") *Eng_cfg >> MaxManifoldPressure_inHg;
-    else if (token == "DISPLACEMENT") *Eng_cfg >> Displacement;
-    else if (token == "MAXHP") *Eng_cfg >> MaxHP;
-    else if (token == "CYCLES") *Eng_cfg >> Cycles;
-    else if (token == "IDLERPM") *Eng_cfg >> IdleRPM;
-    else if (token == "MAXTHROTTLE") *Eng_cfg >> MaxThrottle;
-    else if (token == "MINTHROTTLE") *Eng_cfg >> MinThrottle;
-    else cerr << "Unhandled token in Engine config file: " << token << endl;
-  }
-
   Type = etPiston;
   crank_counter = 0;
   EngineNumber = 0;
   OilTemp_degK = 298;
-  ManifoldPressure_inHg = Atmosphere->GetPressure() * 0.014138; // psf to in Hg
+  MinManifoldPressure_inHg = 6.5;
+  MaxManifoldPressure_inHg = 28.5;
+  ManifoldPressure_inHg = Atmosphere->GetPressure() * psftoinhg; // psf to in Hg
+  CylinderHeadTemp_degK = 0.0;
+  Displacement = 360;
+  MaxHP = 200;
+  Cycles = 2;
+  IdleRPM = 600;
+  Magnetos = 0;
+  ExhaustGasTemp_degK = 0.0;
+  EGT_degC = 0.0;
 
   dt = State->Getdt();
 
@@ -122,6 +111,21 @@ FGPiston::FGPiston(FGFDMExec* exec, FGConfigFile* Eng_cfg) : FGEngine(exec),
   *Power_Mixture_Correlation << 20 <<  74.0;
   *Power_Mixture_Correlation << (14.7/0.6) << 58;
 
+  Name = Eng_cfg->GetValue("NAME");
+  Eng_cfg->GetNextConfigLine();
+  while (Eng_cfg->GetValue() != string("/FG_PISTON")) {
+    *Eng_cfg >> token;
+    if      (token == "MINMP") *Eng_cfg >> MinManifoldPressure_inHg;
+    else if (token == "MAXMP") *Eng_cfg >> MaxManifoldPressure_inHg;
+    else if (token == "DISPLACEMENT") *Eng_cfg >> Displacement;
+    else if (token == "MAXHP") *Eng_cfg >> MaxHP;
+    else if (token == "CYCLES") *Eng_cfg >> Cycles;
+    else if (token == "IDLERPM") *Eng_cfg >> IdleRPM;
+    else if (token == "MAXTHROTTLE") *Eng_cfg >> MaxThrottle;
+    else if (token == "MINTHROTTLE") *Eng_cfg >> MinThrottle;
+    else cerr << "Unhandled token in Engine config file: " << token << endl;
+  }
+
   Debug(0); // Call Debug() routine from constructor if needed
 }
 
@@ -136,7 +140,7 @@ FGPiston::~FGPiston()
 
 double FGPiston::Calculate(double PowerRequired)
 {
-  ConsumeFuel();
+  if (FuelFlow_gph > 0.0) ConsumeFuel();
 
   Throttle = FCS->GetThrottlePos(EngineNumber);
   Mixture = FCS->GetMixturePos(EngineNumber);
@@ -149,8 +153,8 @@ double FGPiston::Calculate(double PowerRequired)
   p_amb_sea_level = Atmosphere->GetPressureSL() * 48;
   T_amb = Atmosphere->GetTemperature() * (5.0 / 9.0);  // convert from Rankine to Kelvin
 
-  RPM = Propulsion->GetThruster(EngineNumber)->GetRPM();
-  //if (RPM < IdleRPM) RPM = IdleRPM;  // kludge
+  RPM = Propulsion->GetThruster(EngineNumber)->GetRPM() *
+        Propulsion->GetThruster(EngineNumber)->GetGearRatio();
     
   IAS = Auxiliary->GetVcalibratedKTS();
 
@@ -163,7 +167,7 @@ double FGPiston::Calculate(double PowerRequired)
   //Assume lean limit at 22 AFR for now - thats a thi of 0.668
   //This might be a bit generous, but since there's currently no audiable warning of impending
   //cutout in the form of misfiring and/or rough running its probably reasonable for now.
-  if(equivalence_ratio < 0.668)
+  if (equivalence_ratio < 0.668)
     Running = false;
 
   doEnginePower();
@@ -216,49 +220,28 @@ void FGPiston::doEngineStartup(void)
     crank_counter = 0;
   }
 
-  //Check mode of engine operation
-  if (Cranking) {
-    crank_counter++;
-    if (RPM <= 480) {
-      // Do nothing !! - cranking power output is now handled in the doPower section
-    } else {
-      // consider making a horrible noise if the starter is engaged with
-      // the engine running
-    }
-  }
+  if (Cranking) crank_counter++;  //Check mode of engine operation
   
-  // if ((!Running) && (spark) && (fuel) && (crank_counter > 120)) {
-
-  if ((!Running) && (spark) && (fuel)) {
-  // start the engine if revs high enough
-    if(Cranking) {
-      if ((RPM > 450) && (crank_counter > 175)) {
-        //Add a little delay to startup on the starter
-        Running = true;
-      }
+  if (!Running && spark && fuel) {  // start the engine if revs high enough
+    if (Cranking) {
+      if ((RPM > 450) && (crank_counter > 175)) // Add a little delay to startup
+        Running = true;                         // on the starter
     } else {
-      if (RPM > 450) {
-        Running = true;
-        //This allows us to in-air start when windmilling
-      }
+      if (RPM > 450)                            // This allows us to in-air start
+        Running = true;                         // when windmilling
     }
   }
 
-  if ( (Running) && ((!spark)||(!fuel)) ) {
-    // Cut the engine
-    // note that we only cut the power - the engine may continue to
-    // spin if the prop is in a moving airstream
-    Running = false;
-  }
+  // Cut the engine *power* - Note: the engine may continue to
+  // spin if the prop is in a moving airstream
 
-  // And finally a last check for stalling
+  if ( Running && (!spark || !fuel) ) Running = false;
+
+  // Check for stalling (RPM = 0).
   if (Running) { 
-    //Check if we have stalled the engine
     if (RPM == 0) {
       Running = false;
     } else if ((RPM <= 480) && (Cranking)) {
-      // Make sure the engine noise dosn't play if the engine won't
-	    // start due to eg mixture lever pulled out.
       Running = false;
     }
   }
@@ -290,7 +273,7 @@ void FGPiston::doManifoldPressure(void)
     ManifoldPressure_inHg = MinManifoldPressure_inHg +
             (Throttle * (MaxManifoldPressure_inHg - MinManifoldPressure_inHg));
   } else {
-    ManifoldPressure_inHg = Atmosphere->GetPressure() * 0.014138; // psf to in Hg
+    ManifoldPressure_inHg = Atmosphere->GetPressure() * psftoinhg; // psf to in Hg
   }  
 }
 
@@ -311,7 +294,7 @@ void FGPiston::doAirFlow(void)
 {
   rho_air = p_amb / (R_air * T_amb);
   double rho_air_manifold = rho_air * ManifoldPressure_inHg / 29.6;
-  double displacement_SI = Displacement * CONVERT_CUBIC_INCHES_TO_METERS_CUBED;
+  double displacement_SI = Displacement * in3tom3;
   double swept_volume = (displacement_SI * (RPM/60)) / 2;
   double v_dot_air = swept_volume * volumetric_efficiency;
   m_dot_air = v_dot_air * rho_air_manifold;
@@ -355,29 +338,34 @@ void FGPiston::doFuelFlow(void)
 void FGPiston::doEnginePower(void)
 {
   ManifoldPressure_inHg *= p_amb / p_amb_sea_level;
-  if(Running) {	
+
+  if (Running) {	
     double ManXRPM = ManifoldPressure_inHg * RPM;
-        // FIXME: this needs to be generalized
+    double T_amb_degF = KelvinToFahrenheit(T_amb);
+    double T_amb_sea_lev_degF = KelvinToFahrenheit(288); 
+
+    // FIXME: this needs to be generalized
     Percentage_Power = (6e-9 * ManXRPM * ManXRPM) + (8e-4 * ManXRPM) - 1.0;
-    double T_amb_degF = (T_amb * 1.8) - 459.67;
-    double T_amb_sea_lev_degF = (288 * 1.8) - 459.67; 
-    Percentage_Power =
-      Percentage_Power + ((T_amb_sea_lev_degF - T_amb_degF) * 7 /120);
+    Percentage_Power += ((T_amb_sea_lev_degF - T_amb_degF) * 7 /120);
+
     double Percentage_of_best_power_mixture_power =
       Power_Mixture_Correlation->GetValue(14.7 / equivalence_ratio);
-    Percentage_Power =
-      Percentage_Power * Percentage_of_best_power_mixture_power / 100.0;
-    if (Percentage_Power < 0.0)
-      Percentage_Power = 0.0;
-    else if (Percentage_Power > 100.0)
-      Percentage_Power = 100.0;
+
+    Percentage_Power *= Percentage_of_best_power_mixture_power / 100.0;
+
+    if (Percentage_Power < 0.0) Percentage_Power = 0.0;
+    else if (Percentage_Power > 100.0) Percentage_Power = 100.0;
+    
     HP = Percentage_Power * MaxHP / 100.0;
-  } else {  
+
+  } else {
+
     // Power output when the engine is not running
     if (Cranking) {
       if (RPM < 10) {
-        HP = 3.0;	// This is a hack to prevent overshooting the idle rpm in the first time step
-                    // It may possibly need to be changed if the prop model is changed.
+        HP = 3.0;   // This is a hack to prevent overshooting the idle rpm in
+                    // the first time step. It may possibly need to be changed
+                    // if the prop model is changed.
       } else if (RPM < 480) {
         HP = 3.0 + ((480 - RPM) / 10.0);  
         // This is a guess - would be nice to find a proper starter moter torque curve
@@ -459,7 +447,8 @@ void FGPiston::doCHT(void)
     
   double HeatCapacityCylinderHead = CpCylinderHead * MassCylinderHead;
     
-  CylinderHeadTemp_degK = dqdt_cylinder_head / HeatCapacityCylinderHead;
+  CylinderHeadTemp_degK +=
+    (dqdt_cylinder_head / HeatCapacityCylinderHead) * dt;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -507,8 +496,7 @@ void FGPiston::doOilPressure(void)
 {
   double Oil_Press_Relief_Valve = 60; // FIXME: may vary by engine
   double Oil_Press_RPM_Max = 1800;    // FIXME: may vary by engine
-  double Design_Oil_Temp = 85;        // FIXME: may vary by engine
-             // FIXME: WRONG!!! (85 degK???)
+  double Design_Oil_Temp = 358;	      // degK; FIXME: may vary by engine
   double Oil_Viscosity_Index = 0.25;
 
   OilPressure_psi = (Oil_Press_Relief_Valve / Oil_Press_RPM_Max) * RPM;
@@ -590,5 +578,7 @@ void FGPiston::Debug(int from)
 double
 FGPiston::CalcFuelNeed(void)
 {
-  return FuelFlow_gph / 3600 * State->Getdt() * Propulsion->GetRate();
+  return FuelFlow_gph / 3600 * 6 * State->Getdt() * Propulsion->GetRate();
 }
+
+} // namespace JSBSim
