@@ -53,10 +53,31 @@ INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #include "FGPropulsion.h"
+#include "FGRocket.h"
+#include "FGSimTurbine.h"
+#include "FGTurbine.h"
+#include "FGPropeller.h"
+#include "FGNozzle.h"
+#include "FGPiston.h"
+#include "FGPropertyManager.h"
 
-static const char *IdSrc = "$Id: FGPropulsion.cpp,v 1.29 2001/12/24 12:54:56 david Exp $";
+#if defined (__APPLE__)
+/* Not all systems have the gcvt function */
+inline char* gcvt (double value, int ndigits, char *buf) {
+    /* note that this is not exactly what gcvt is supposed to do! */
+    snprintf (buf, ndigits+1, "%f", value);
+    return buf;
+}
+#endif
+
+namespace JSBSim {
+
+static const char *IdSrc = "$Id: FGPropulsion.cpp,v 1.9 2004/03/14 14:57:07 ehofman Exp $";
 static const char *IdHdr = ID_PROPULSION;
 
+extern short debug_lvl;
+
+
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -64,9 +85,15 @@ CLASS IMPLEMENTATION
 FGPropulsion::FGPropulsion(FGFDMExec* exec) : FGModel(exec)
 {
   Name = "FGPropulsion";
+
   numSelectedFuelTanks = numSelectedOxiTanks = 0;
   numTanks = numEngines = numThrusters = 0;
   numOxiTanks = numFuelTanks = 0;
+  dt = 0.0;
+  ActiveEngine = -1; // -1: ALL, 0: Engine 1, 1: Engine 2 ...
+  tankJ.InitMatrix();
+
+  bind();
 
   Debug(0);
 }
@@ -77,6 +104,7 @@ FGPropulsion::~FGPropulsion()
 {
   for (unsigned int i=0; i<Engines.size(); i++) delete Engines[i];
   Engines.clear();
+  unbind();
   Debug(1);
 }
 
@@ -98,6 +126,7 @@ bool FGPropulsion::Run(void)
       vForces  += Thrusters[i]->GetBodyForces();  // sum body frame forces
       vMoments += Thrusters[i]->GetMoments();     // sum body frame moments
     }
+
     return false;
   } else {
     return true;
@@ -127,13 +156,13 @@ bool FGPropulsion::GetSteadyState(void)
         PowerAvailable = Engines[i]->Calculate(Thrusters[i]->GetPowerRequired());
         lastThrust = currentThrust;
         currentThrust = Thrusters[i]->Calculate(PowerAvailable);
-        if(fabs(lastThrust-currentThrust) < 0.0001) {
+        if (fabs(lastThrust-currentThrust) < 0.0001) {
           steady_count++;
-          if(steady_count > 120) { steady=true; }
+          if (steady_count > 120) { steady=true; }
         } else {
           steady_count=0;
         }
-        j++;    
+        j++;
       }
       vForces  += Thrusters[i]->GetBodyForces();  // sum body frame forces
       vMoments += Thrusters[i]->GetMoments();     // sum body frame moments
@@ -156,7 +185,7 @@ bool FGPropulsion::ICEngineStart(void)
 
   vForces.InitMatrix();
   vMoments.InitMatrix();
-    
+
   for (unsigned int i=0; i<numEngines; i++) {
     Engines[i]->SetTrimMode(true);
     Thrusters[i]->SetdeltaT(dt*rate);
@@ -164,7 +193,7 @@ bool FGPropulsion::ICEngineStart(void)
     while (!Engines[i]->GetRunning() && j < 2000) {
       PowerAvailable = Engines[i]->Calculate(Thrusters[i]->GetPowerRequired());
       Thrusters[i]->Calculate(PowerAvailable);
-      j++;    
+      j++;
     }
     vForces  += Thrusters[i]->GetBodyForces();  // sum body frame forces
     vMoments += Thrusters[i]->GetMoments();     // sum body frame moments
@@ -216,12 +245,10 @@ bool FGPropulsion::Load(FGConfigFile* AC_cfg)
           Engines.push_back(new FGRocket(FDMExec, &Eng_cfg));
         } else if (engType == "FG_PISTON") {
           Engines.push_back(new FGPiston(FDMExec, &Eng_cfg));
-        } else if (engType == "FG_TURBOJET") {
-          Engines.push_back(new FGTurboJet(FDMExec, &Eng_cfg));
-        } else if (engType == "FG_TURBOSHAFT") {
-          Engines.push_back(new FGTurboShaft(FDMExec, &Eng_cfg));
-        } else if (engType == "FG_TURBOPROP") {
-          Engines.push_back(new FGTurboProp(FDMExec, &Eng_cfg));
+        } else if (engType == "FG_TURBINE") {
+          Engines.push_back(new FGTurbine(FDMExec, &Eng_cfg));
+        } else if (engType == "FG_SIMTURBINE") {
+          Engines.push_back(new FGSimTurbine(FDMExec, &Eng_cfg));
         } else {
           cerr << fgred << "    Unrecognized engine type: " << underon << engType
                     << underoff << " found in config file." << fgdef << endl;
@@ -295,7 +322,9 @@ bool FGPropulsion::Load(FGConfigFile* AC_cfg)
         if (thrType == "FG_PROPELLER") {
           Thrusters.push_back(new FGPropeller(FDMExec, &Thruster_cfg));
         } else if (thrType == "FG_NOZZLE") {
-          Thrusters.push_back(new FGNozzle(FDMExec, &Thruster_cfg));
+          Thrusters.push_back(new FGNozzle(FDMExec, &Thruster_cfg ));
+        } else if (thrType == "FG_DIRECT") {
+          Thrusters.push_back(new FGThruster( FDMExec, &Thruster_cfg) );
         }
 
         AC_cfg->GetNextConfigLine();
@@ -334,6 +363,7 @@ bool FGPropulsion::Load(FGConfigFile* AC_cfg)
     AC_cfg->GetNextConfigLine();
   }
 
+  CalculateTankInertias();
   if (!ThrottleAdded) FCS->AddThrottle(); // need to have at least one throttle
 
   return true;
@@ -360,9 +390,11 @@ string FGPropulsion::GetPropulsionStrings(void)
     case FGEngine::etRocket:
       PropulsionStrings += (Engines[i]->GetName() + "_ChamberPress[" + buffer + "]");
       break;
-    case FGEngine::etTurboJet:
-    case FGEngine::etTurboProp:
-    case FGEngine::etTurboShaft:
+    case FGEngine::etTurbine:
+      break;
+    case FGEngine::etSimTurbine:
+      PropulsionStrings += (Engines[i]->GetName() + "_N1[" + buffer + "], ");
+      PropulsionStrings += (Engines[i]->GetName() + "_N2[" + buffer + "]");
       break;
     default:
       PropulsionStrings += "INVALID ENGINE TYPE";
@@ -388,10 +420,13 @@ string FGPropulsion::GetPropulsionStrings(void)
         PropulsionStrings += (Thrusters[i]->GetName() + "_Pitch[" + buffer + "], ");
       PropulsionStrings += (Thrusters[i]->GetName() + "_RPM[" + buffer + "]");
       break;
+    case FGThruster::ttDirect:
+      PropulsionStrings += (Thrusters[i]->GetName() + "_Thrust[" + buffer + "]");
+      break;
     default:
       PropulsionStrings += "INVALID THRUSTER TYPE";
       break;
-    }    
+    }
   }
 
   return PropulsionStrings;
@@ -416,9 +451,11 @@ string FGPropulsion::GetPropulsionValues(void)
     case FGEngine::etRocket:
       PropulsionValues += (string(gcvt(((FGRocket*)Engines[i])->GetChamberPressure(), 10, buff)));
       break;
-    case FGEngine::etTurboJet:
-    case FGEngine::etTurboProp:
-    case FGEngine::etTurboShaft:
+    case FGEngine::etTurbine:
+      break;
+    case FGEngine::etSimTurbine:
+      PropulsionValues += (string(gcvt(((FGSimTurbine*)Engines[i])->GetN1(), 10, buff))) + ", ";
+      PropulsionValues += (string(gcvt(((FGSimTurbine*)Engines[i])->GetN2(), 10, buff)));
       break;
     }
 
@@ -430,6 +467,9 @@ string FGPropulsion::GetPropulsionValues(void)
       break;
     case FGThruster::ttRotor:
       break;
+    case FGThruster::ttDirect:
+      PropulsionValues += (string(gcvt(((FGThruster*)Thrusters[i])->GetThrust(), 10, buff)));
+      break;
     case FGThruster::ttPropeller:
       FGPropeller* Propeller = (FGPropeller*)Thrusters[i];
       FGColumnVector3 vPFactor = Propeller->GetPFactor();
@@ -450,17 +490,17 @@ string FGPropulsion::GetPropulsionValues(void)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGColumnVector3& FGPropulsion::GetTanksCG(void)
+FGColumnVector3& FGPropulsion::GetTanksMoment(void)
 {
   iTank = Tanks.begin();
-  vXYZtank.InitMatrix();
+  vXYZtank_arm.InitMatrix();
   while (iTank < Tanks.end()) {
-    vXYZtank(eX) += (*iTank)->GetX()*(*iTank)->GetContents();
-    vXYZtank(eY) += (*iTank)->GetY()*(*iTank)->GetContents();
-    vXYZtank(eZ) += (*iTank)->GetZ()*(*iTank)->GetContents();
+    vXYZtank_arm(eX) += (*iTank)->GetXYZ(eX)*(*iTank)->GetContents();
+    vXYZtank_arm(eY) += (*iTank)->GetXYZ(eY)*(*iTank)->GetContents();
+    vXYZtank_arm(eZ) += (*iTank)->GetXYZ(eZ)*(*iTank)->GetContents();
     iTank++;
   }
-  return vXYZtank;
+  return vXYZtank_arm;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -479,67 +519,128 @@ double FGPropulsion::GetTanksWeight(void)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-double FGPropulsion::GetTanksIxx(const FGColumnVector3& vXYZcg)
+FGMatrix33& FGPropulsion::CalculateTankInertias(void)
 {
-  double I = 0.0;
-  iTank = Tanks.begin();
-  while (iTank < Tanks.end()) {
-    I += ((*iTank)->GetX() - vXYZcg(eX))*((*iTank)->GetX() - vXYZcg(eX)) * (*iTank)->GetContents()/(144.0*Inertial->gravity());
-    iTank++;
-  }
-  return I;
+  unsigned int size;
+
+  size = Tanks.size();
+  if (size == 0) return tankJ;
+
+  tankJ = FGMatrix33();
+
+  for (unsigned int i=0; i<size; i++)
+    tankJ += MassBalance->GetPointmassInertia( lbtoslug * Tanks[i]->GetContents(),
+                                               Tanks[i]->GetXYZ() );
+
+  return tankJ;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-double FGPropulsion::GetTanksIyy(const FGColumnVector3& vXYZcg)
+void FGPropulsion::SetMagnetos(int setting)
 {
-  double I = 0.0;
-  iTank = Tanks.begin();
-  while (iTank < Tanks.end()) {
-    I += ((*iTank)->GetY() - vXYZcg(eY))*((*iTank)->GetY() - vXYZcg(eY)) * (*iTank)->GetContents()/(144.0*Inertial->gravity());
-    iTank++;
+  if (ActiveEngine < 0) {
+    for (unsigned i=0; i<Engines.size(); i++) {
+      ((FGPiston*)Engines[i])->SetMagnetos(setting);
+    }
+  } else {
+    ((FGPiston*)Engines[ActiveEngine])->SetMagnetos(setting);
   }
-  return I;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-double FGPropulsion::GetTanksIzz(const FGColumnVector3& vXYZcg)
+void FGPropulsion::SetStarter(int setting)
 {
-  double I = 0.0;
-  iTank = Tanks.begin();
-  while (iTank < Tanks.end()) {
-    I += ((*iTank)->GetZ() - vXYZcg(eZ))*((*iTank)->GetZ() - vXYZcg(eZ)) * (*iTank)->GetContents()/(144.0*Inertial->gravity());
-    iTank++;
+  if (ActiveEngine < 0) {
+    for (unsigned i=0; i<Engines.size(); i++) {
+      if (setting == 0)
+        Engines[i]->SetStarter(false);
+      else
+        Engines[i]->SetStarter(true);
+    }
+  } else {
+    if (setting == 0)
+      Engines[ActiveEngine]->SetStarter(false);
+    else
+      Engines[ActiveEngine]->SetStarter(true);
   }
-  return I;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-double FGPropulsion::GetTanksIxz(const FGColumnVector3& vXYZcg)
+void FGPropulsion::SetCutoff(int setting)
 {
-  double I = 0.0;
-  iTank = Tanks.begin();
-  while (iTank < Tanks.end()) {
-    I += ((*iTank)->GetX() - vXYZcg(eX))*((*iTank)->GetZ() - vXYZcg(eZ)) * (*iTank)->GetContents()/(144.0*Inertial->gravity());
-    iTank++;
+  if (ActiveEngine < 0) {
+    for (unsigned i=0; i<Engines.size(); i++) {
+      if (setting == 0)
+        ((FGSimTurbine*)Engines[i])->SetCutoff(false);
+      else
+        ((FGSimTurbine*)Engines[i])->SetCutoff(true);
+    }
+  } else {
+    if (setting == 0)
+      ((FGSimTurbine*)Engines[ActiveEngine])->SetCutoff(false);
+    else
+      ((FGSimTurbine*)Engines[ActiveEngine])->SetCutoff(true);
   }
-  return I;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-double FGPropulsion::GetTanksIxy(const FGColumnVector3& vXYZcg)
+void FGPropulsion::SetActiveEngine(int engine)
 {
-  double I = 0.0;
-  iTank = Tanks.begin();
-  while (iTank != Tanks.end()) {
-    I += ((*iTank)->GetX() - vXYZcg(eX))*((*iTank)->GetY() - vXYZcg(eY)) * (*iTank)->GetContents()/(144.0*Inertial->gravity());
-    iTank++;
-  }
-  return I;
+  if (engine >= Engines.size() || engine < 0)
+    ActiveEngine = -1;
+  else
+    ActiveEngine = engine;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGPropulsion::bind(void)
+{
+  typedef double (FGPropulsion::*PMF)(int) const;
+  typedef int (FGPropulsion::*iPMF)(void) const;
+
+  PropertyManager->Tie("propulsion/magneto_cmd", this,
+                       (iPMF)0, &FGPropulsion::SetMagnetos, true);
+  PropertyManager->Tie("propulsion/starter_cmd", this,
+                       (iPMF)0, &FGPropulsion::SetStarter,  true);
+  PropertyManager->Tie("propulsion/cutoff_cmd", this,
+                       (iPMF)0, &FGPropulsion::SetCutoff,   true);
+
+  PropertyManager->Tie("forces/fbx-prop-lbs", this,1,
+                       (PMF)&FGPropulsion::GetForces);
+  PropertyManager->Tie("forces/fby-prop-lbs", this,2,
+                       (PMF)&FGPropulsion::GetForces);
+  PropertyManager->Tie("forces/fbz-prop-lbs", this,3,
+                       (PMF)&FGPropulsion::GetForces);
+  PropertyManager->Tie("moments/l-prop-lbsft", this,1,
+                       (PMF)&FGPropulsion::GetMoments);
+  PropertyManager->Tie("moments/m-prop-lbsft", this,2,
+                       (PMF)&FGPropulsion::GetMoments);
+  PropertyManager->Tie("moments/n-prop-lbsft", this,3,
+                       (PMF)&FGPropulsion::GetMoments);
+
+  PropertyManager->Tie("propulsion/active_engine", this,
+           (iPMF)&FGPropulsion::GetActiveEngine, &FGPropulsion::SetActiveEngine, true);
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGPropulsion::unbind(void)
+{
+  PropertyManager->Untie("propulsion/magneto_cmd");
+  PropertyManager->Untie("propulsion/starter_cmd");
+  PropertyManager->Untie("propulsion/cutoff_cmd");
+  PropertyManager->Untie("propulsion/active_engine");
+  PropertyManager->Untie("forces/fbx-prop-lbs");
+  PropertyManager->Untie("forces/fby-prop-lbs");
+  PropertyManager->Untie("forces/fbz-prop-lbs");
+  PropertyManager->Untie("moments/l-prop-lbsft");
+  PropertyManager->Untie("moments/m-prop-lbsft");
+  PropertyManager->Untie("moments/n-prop-lbsft");
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -587,4 +688,4 @@ void FGPropulsion::Debug(int from)
     }
   }
 }
-
+}
