@@ -12,12 +12,20 @@ public:
     void setCompression(float c);
 
     void setThrottle(float throttle);
+    void setStarter(bool starter);
+    void setMagnetos(int magnetos);
     void setMixture(float mixture);
     void setBoost(float boost); // fraction of turbo-mul used
+    void setFuelState(bool hasFuel) { _fuel = hasFuel; }
+
+    // For solver use
+    void setRunning(bool r);
 
     float getMaxPower(); // max sea-level power
 
     void calc(float pressure, float temp, float speed);
+    bool isRunning();
+    bool isCranking();
     float getTorque();
     float getFuelFlow();
     float getMP();
@@ -37,10 +45,15 @@ private:
 
     // Runtime settables:
     float _throttle;
+    bool _starter; // true=engaged, false=disengaged
+    int _magnetos; // 0=off, 1=right, 2=left, 3=both
     float _mixture;
     float _boost;
+    bool _fuel;
 
     // Runtime state/output:
+    bool _running;
+    bool _cranking;
     float _mp;
     float _torque;
     float _fuelFlow;
