@@ -9,6 +9,7 @@ Thruster::Thruster()
     for(i=0; i<3; i++) _pos[i] = _wind[i] = 0;
     _throttle = 0;
     _mixture = 0;
+    _starter = false;
     _pressure = _temp = _rho = 0;
 }
 
@@ -49,17 +50,22 @@ void Thruster::setMixture(float mixture)
     _mixture = Math::clamp(mixture, 0, 1);
 }
 
+void Thruster::setStarter(bool starter)
+{
+    _starter = starter;
+}
+
 void Thruster::setWind(float* wind)
 {
     int i;
     for(i=0; i<3; i++) _wind[i] = wind[i];
 }
 
-void Thruster::setAir(float pressure, float temp)
+void Thruster::setAir(float pressure, float temp, float density)
 {
     _pressure = pressure;
     _temp = temp;
-    _rho = _pressure / (287.1 * _temp);
+    _rho = density;
 }
 
 }; // namespace yasim
