@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: flight.cxx,v 1.53 2002/02/16 16:38:29 curt Exp $
+// $Id: flight.cxx,v 1.16 2004/03/19 03:30:18 curt Exp $
 
 
 #include <stdio.h>
@@ -28,6 +28,7 @@
 #include <simgear/constants.h>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/math/sg_geodesy.hxx>
+#include <simgear/scene/model/placement.hxx>
 #include <simgear/timing/timestamp.hxx>
 
 #include <Scenery/scenery.hxx>
@@ -35,11 +36,7 @@
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 
-#include "External.hxx"
 #include "flight.hxx"
-#include "JSBSim.hxx"
-#include "LaRCsim.hxx"
-#include "Balloon.h"
 
 
 // base_fdm_state is the internal state that is updated in integer
@@ -55,14 +52,16 @@ inline void init_vec(FG_VECTOR_3 vec) {
 }
 
 // Constructor
-FGInterface::FGInterface() {
+FGInterface::FGInterface()
+  : remainder(0)
+{
     _setup();
 }
 
-FGInterface::FGInterface( double dt ) {
+FGInterface::FGInterface( double dt )
+  : remainder(0)
+{
     _setup();
-    delta_t = dt;
-    remainder = elapsed = multi_loop = 0;
 }
 
 // Destructor
@@ -71,6 +70,21 @@ FGInterface::~FGInterface() {
 }
 
 
+int
+FGInterface::_calc_multiloop (double dt)
+{
+  int hz = fgGetInt("/sim/model-hz");
+  int speedup = fgGetInt("/sim/speed-up");
+
+  dt += remainder;
+  remainder = 0;
+  double ml = dt * hz;
+  int multiloop = int(floor(ml));
+  remainder = (ml - multiloop) / hz;
+  return (multiloop * speedup);
+}
+
+
 /**
  * Set default values for the state of the FDM.
  *
@@ -167,68 +181,67 @@ FGInterface::common_init ()
 
     set_inited( true );
 
-    stamp();
-    set_remainder( 0 );
+//     stamp();
+//     set_remainder( 0 );
 
     // Set initial position
     SG_LOG( SG_FLIGHT, SG_INFO, "...initializing position..." );
-    set_Longitude( fgGetDouble("/position/longitude-deg")
+    set_Longitude( fgGetDouble("/sim/presets/longitude-deg")
                    * SGD_DEGREES_TO_RADIANS );
-    set_Latitude( fgGetDouble("/position/latitude-deg")
+    set_Latitude( fgGetDouble("/sim/presets/latitude-deg")
                   * SGD_DEGREES_TO_RADIANS );
-    double ground_elev_m = scenery.get_cur_elev();
+    double ground_elev_m = globals->get_scenery()->get_cur_elev();
     double ground_elev_ft = ground_elev_m * SG_METER_TO_FEET;
-    if ( fgGetBool("/sim/startup/onground")
-         || fgGetDouble("/position/altitude-ft") < ground_elev_ft ) {
+    fgSetDouble("/position/ground-elev-m", ground_elev_m);
+    _set_Runway_altitude ( ground_elev_ft );
+    if ( fgGetBool("/sim/presets/onground")
+         || fgGetDouble("/sim/presets/altitude-ft") < ground_elev_ft ) {
+        fgSetDouble("/sim/presets/altitude-ft", ground_elev_ft);
         fgSetDouble("/position/altitude-ft", ground_elev_ft);
     }
-    set_Altitude( fgGetDouble("/position/altitude-ft") );
+    set_Altitude( fgGetDouble("/sim/presets/altitude-ft") );
 
     // Set ground elevation
     SG_LOG( SG_FLIGHT, SG_INFO,
             "...initializing ground elevation to " << ground_elev_ft
             << "ft..." );
-    SG_LOG( SG_FLIGHT, SG_INFO, "common_init(): set ground elevation "
-            << ground_elev_ft );
-    base_fdm_state.set_Runway_altitude( ground_elev_ft );
-    set_Runway_altitude( ground_elev_ft );
 
     // Set sea-level radius
     SG_LOG( SG_FLIGHT, SG_INFO, "...initializing sea-level radius..." );
     SG_LOG( SG_FLIGHT, SG_INFO, " lat = "
-            << fgGetDouble("/position/latitude-deg")
-            << " alt = " << fgGetDouble("/position/altitude-ft") );
+            << fgGetDouble("/sim/presets/latitude-deg")
+            << " alt = " << fgGetDouble("/sim/presets/altitude-ft") );
     double sea_level_radius_meters;
     double lat_geoc;
-    sgGeodToGeoc( fgGetDouble("/position/latitude-deg")
+    sgGeodToGeoc( fgGetDouble("/sim/presets/latitude-deg")
                     * SGD_DEGREES_TO_RADIANS,
-                  fgGetDouble("/position/altitude-ft") * SG_FEET_TO_METER,
+                  fgGetDouble("/sim/presets/altitude-ft") * SG_FEET_TO_METER,
                   &sea_level_radius_meters, &lat_geoc );
-    set_Sea_level_radius( sea_level_radius_meters * SG_METER_TO_FEET );
+    _set_Sea_level_radius( sea_level_radius_meters * SG_METER_TO_FEET );
 
     // Set initial velocities
     SG_LOG( SG_FLIGHT, SG_INFO, "...initializing velocities..." );
-    if ( !fgHasNode("/sim/startup/speed-set") ) {
+    if ( !fgHasNode("/sim/presets/speed-set") ) {
         set_V_calibrated_kts(0.0);
     } else {
-        const string speedset = fgGetString("/sim/startup/speed-set");
+        const string speedset = fgGetString("/sim/presets/speed-set");
         if ( speedset == "knots" || speedset == "KNOTS" ) {
-            set_V_calibrated_kts( fgGetDouble("/velocities/airspeed-kt") );
+            set_V_calibrated_kts( fgGetDouble("/sim/presets/airspeed-kt") );
         } else if ( speedset == "mach" || speedset == "MACH" ) {
-            set_Mach_number( fgGetDouble("/velocities/mach") );
+            set_Mach_number( fgGetDouble("/sim/presets/mach") );
         } else if ( speedset == "UVW" || speedset == "uvw" ) {
             set_Velocities_Wind_Body(
-                                     fgGetDouble("/velocities/uBody-fps"),
-                                     fgGetDouble("/velocities/vBody-fps"),
-                                     fgGetDouble("/velocities/wBody-fps") );
+                                     fgGetDouble("/sim/presets/uBody-fps"),
+                                     fgGetDouble("/sim/presets/vBody-fps"),
+                                     fgGetDouble("/sim/presets/wBody-fps") );
         } else if ( speedset == "NED" || speedset == "ned" ) {
             set_Velocities_Local(
-                                 fgGetDouble("/velocities/speed-north-fps"),
-                                 fgGetDouble("/velocities/speed-east-fps"),
-                                 fgGetDouble("/velocities/speed-down-fps") );
+                                 fgGetDouble("/sim/presets/speed-north-fps"),
+                                 fgGetDouble("/sim/presets/speed-east-fps"),
+                                 fgGetDouble("/sim/presets/speed-down-fps") );
         } else {
             SG_LOG( SG_FLIGHT, SG_ALERT,
-                    "Unrecognized value for /sim/startup/speed-set: "
+                    "Unrecognized value for /sim/presets/speed-set: "
                     << speedset);
             set_V_calibrated_kts( 0.0 );
         }
@@ -236,11 +249,11 @@ FGInterface::common_init ()
 
     // Set initial Euler angles
     SG_LOG( SG_FLIGHT, SG_INFO, "...initializing Euler angles..." );
-    set_Euler_Angles( fgGetDouble("/orientation/roll-deg")
+    set_Euler_Angles( fgGetDouble("/sim/presets/roll-deg")
                         * SGD_DEGREES_TO_RADIANS,
-                      fgGetDouble("/orientation/pitch-deg")
+                      fgGetDouble("/sim/presets/pitch-deg")
                         * SGD_DEGREES_TO_RADIANS,
-                      fgGetDouble("/orientation/heading-deg")
+                      fgGetDouble("/sim/presets/heading-deg")
                         * SGD_DEGREES_TO_RADIANS );
 
     SG_LOG( SG_FLIGHT, SG_INFO, "End common FDM init" );
@@ -262,14 +275,14 @@ FGInterface::bind ()
   bound = true;
 
                                 // Time management (read-only)
-  fgTie("/fdm/time/delta_t", this,
-        &FGInterface::get_delta_t); // read-only
-  fgTie("/fdm/time/elapsed", this,
-        &FGInterface::get_elapsed); // read-only
-  fgTie("/fdm/time/remainder", this,
-        &FGInterface::get_remainder); // read-only
-  fgTie("/fdm/time/multi_loop", this,
-        &FGInterface::get_multi_loop); // read-only
+//   fgTie("/fdm/time/delta_t", this,
+//         &FGInterface::get_delta_t); // read-only
+//   fgTie("/fdm/time/elapsed", this,
+//         &FGInterface::get_elapsed); // read-only
+//   fgTie("/fdm/time/remainder", this,
+//         &FGInterface::get_remainder); // read-only
+//   fgTie("/fdm/time/multi_loop", this,
+//         &FGInterface::get_multi_loop); // read-only
 
 			// Aircraft position
   fgTie("/position/latitude-deg", this,
@@ -304,10 +317,25 @@ FGInterface::bind ()
 	&FGInterface::set_Psi_deg);
   fgSetArchivable("/orientation/heading-deg");
 
+  // Body-axis "euler rates" (rotation speed, but in a funny
+  // representation).
+  fgTie("/orientation/roll-rate-degps", this,
+	&FGInterface::get_Phi_dot_degps);
+  fgTie("/orientation/pitch-rate-degps", this,
+	&FGInterface::get_Theta_dot_degps);
+  fgTie("/orientation/yaw-rate-degps", this,
+	&FGInterface::get_Psi_dot_degps);
+
 				// Calibrated airspeed
   fgTie("/velocities/airspeed-kt", this,
 	&FGInterface::get_V_calibrated_kts,
 	&FGInterface::set_V_calibrated_kts,
+	false);
+
+				// Mach number
+  fgTie("/velocities/mach", this,
+	&FGInterface::get_Mach_number,
+	&FGInterface::set_Mach_number,
 	false);
 
 				// Local velocities
@@ -355,9 +383,36 @@ FGInterface::bind ()
 
 				// Climb and slip (read-only)
   fgTie("/velocities/vertical-speed-fps", this,
-	&FGInterface::get_Climb_Rate); // read-only
-  fgTie("/velocities/side-slip-rad", this,
+	&FGInterface::get_Climb_Rate,
+  &FGInterface::set_Climb_Rate ); 
+  fgTie("/velocities/glideslope", this,
+  &FGInterface::get_Gamma_vert_rad,
+  &FGInterface::set_Gamma_vert_rad );
+  fgTie("/orientation/side-slip-rad", this,
 	&FGInterface::get_Beta); // read-only
+  fgTie("/orientation/side-slip-deg", this,
+  &FGInterface::get_Beta_deg); // read-only
+  fgTie("/orientation/alpha-deg", this,
+  &FGInterface::get_Alpha_deg); // read-only
+  fgTie("/accelerations/nlf", this,
+  &FGInterface::get_Nlf); // read-only
+
+                                // NED accelerations
+  fgTie("/accelerations/ned/north-accel-fps_sec",
+        this, &FGInterface::get_V_dot_north);
+  fgTie("/accelerations/ned/east-accel-fps_sec",
+        this, &FGInterface::get_V_dot_east);
+  fgTie("/accelerations/ned/down-accel-fps_sec",
+        this, &FGInterface::get_V_dot_down);
+
+                                // Pilot accelerations
+  fgTie("/accelerations/pilot/x-accel-fps_sec",
+        this, &FGInterface::get_A_X_pilot);
+  fgTie("/accelerations/pilot/y-accel-fps_sec",
+        this, &FGInterface::get_A_Y_pilot);
+  fgTie("/accelerations/pilot/z-accel-fps_sec",
+        this, &FGInterface::get_A_Z_pilot);
+
 }
 
 
@@ -372,18 +427,25 @@ FGInterface::unbind ()
 {
   bound = false;
 
-  fgUntie("/fdm/time/delta_t");
-  fgUntie("/fdm/time/elapsed");
-  fgUntie("/fdm/time/remainder");
-  fgUntie("/fdm/time/multi_loop");
+  // fgUntie("/fdm/time/delta_t");
+  // fgUntie("/fdm/time/elapsed");
+  // fgUntie("/fdm/time/remainder");
+  // fgUntie("/fdm/time/multi_loop");
   fgUntie("/position/latitude-deg");
   fgUntie("/position/longitude-deg");
   fgUntie("/position/altitude-ft");
   fgUntie("/position/altitude-agl-ft");
-  fgUntie("/orientation/heading-deg");
-  fgUntie("/orientation/pitch-deg");
   fgUntie("/orientation/roll-deg");
+  fgUntie("/orientation/pitch-deg");
+  fgUntie("/orientation/heading-deg");
+  fgUntie("/orientation/roll-rate-degps");
+  fgUntie("/orientation/pitch-rate-degps");
+  fgUntie("/orientation/yaw-rate-degps");
+  fgUntie("/orientation/side-slip-rad");
+  fgUntie("/orientation/side-slip-deg");
+  fgUntie("/orientation/alpha-deg");
   fgUntie("/velocities/airspeed-kt");
+  fgUntie("/velocities/mach");
   fgUntie("/velocities/speed-north-fps");
   fgUntie("/velocities/speed-east-fps");
   fgUntie("/velocities/speed-down-fps");
@@ -391,16 +453,23 @@ FGInterface::unbind ()
   fgUntie("/velocities/vBody-fps");
   fgUntie("/velocities/wBody-fps");
   fgUntie("/velocities/vertical-speed-fps");
-  fgUntie("/velocities/side-slip-rad");
+  fgUntie("/velocities/glideslope");
+  fgUntie("/accelerations/nlf");
+  fgUntie("/accelerations/pilot/x-accel-fps_sec");
+  fgUntie("/accelerations/pilot/y-accel-fps_sec");
+  fgUntie("/accelerations/pilot/z-accel-fps_sec");
+  fgUntie("/accelerations/ned/north-accel-fps_sec");
+  fgUntie("/accelerations/ned/east-accel-fps_sec");
+  fgUntie("/accelerations/ned/down-accel-fps_sec");
 }
 
 /**
  * Update the state of the FDM (i.e. run the equations of motion).
  */
 void
-FGInterface::update (int dt)
+FGInterface::update (double dt)
 {
-    cout << "dummy update() ... SHOULDN'T BE CALLED!" << endl;
+    SG_LOG(SG_FLIGHT, SG_ALERT, "dummy update() ... SHOULDN'T BE CALLED!");
 }
 
 
@@ -425,7 +494,7 @@ void FGInterface::_updateGeodeticPosition( double lat, double lon, double alt )
     _set_Geodetic_Position( lat, lon, alt );
 
     _set_Sea_level_radius( sl_radius * SG_METER_TO_FEET );
-    _set_Runway_altitude( scenery.get_cur_elev() * SG_METER_TO_FEET );
+    _set_Runway_altitude( fgGetDouble("/position/ground-elev-m") * SG_METER_TO_FEET );
 
     _set_sin_lat_geocentric( lat_geoc );
     _set_cos_lat_geocentric( lat_geoc );
@@ -481,7 +550,7 @@ void FGInterface::_updateGeocentricPosition( double lat_geoc, double lon,
     _set_Geodetic_Position( lat_geod, lon, alt );
 
     _set_Sea_level_radius( sl_radius2 * SG_METER_TO_FEET );
-    _set_Runway_altitude( scenery.get_cur_elev() * SG_METER_TO_FEET );
+    _set_Runway_altitude(  fgGetDouble("/position/ground-elev-m") * SG_METER_TO_FEET );
 
     _set_sin_lat_geocentric( lat_geoc );
     _set_cos_lat_geocentric( lat_geoc );
@@ -518,7 +587,7 @@ void FGInterface::extrapolate( int time_offset ) {
     // -dw- metrowerks complains about ambiguous access, not critical
     // to keep this ;)
 #ifndef __MWERKS__
-    cout << "extrapolating FDM by dt = " << dt << endl;
+    SG_LOG(SG_FLIGHT, SG_INFO, "extrapolating FDM by dt = " << dt);
 #endif
 
     double lat = geodetic_position_v[0] + geocentric_rates_v[0] * dt;
@@ -540,35 +609,6 @@ void FGInterface::extrapolate( int time_offset ) {
     geocentric_position_v[2] = radius;
 }
 
-
-// Set the altitude (force)
-void fgFDMForceAltitude(const string &model, double alt_meters) {
-    SG_LOG(SG_FLIGHT,SG_INFO, "fgFDMForceAltitude: " << alt_meters );
-
-    double sea_level_radius_meters;
-    double lat_geoc;
-
-    // Set the FG variables first
-    sgGeodToGeoc( base_fdm_state.get_Latitude(), alt_meters, 
-		  &sea_level_radius_meters, &lat_geoc);
-
-    base_fdm_state.set_Altitude( alt_meters * SG_METER_TO_FEET );
-    base_fdm_state.set_Sea_level_radius( sea_level_radius_meters *
-					 SG_METER_TO_FEET ); 
-		
-    cur_fdm_state->set_Altitude( alt_meters * SG_METER_TO_FEET );
-    cur_fdm_state->set_Sea_level_radius( sea_level_radius_meters * 
-                                         SG_METER_TO_FEET );
-    cur_fdm_state->set_Runway_altitude( scenery.get_cur_elev() *
-					SG_METER_TO_FEET );	
-
-    // additional work needed for some flight models
-    if ( model == "larcsim" ) {
-	ls_ForceAltitude( base_fdm_state.get_Altitude() );
-    }
-}
-
-
 // Positions
 void FGInterface::set_Latitude(double lat) {
     geodetic_position_v[0] = lat;
@@ -629,15 +669,6 @@ void FGInterface::set_Gamma_vert_rad( double gamma) {
     gamma_vert_rad = gamma;
 }
 
-// Earth
-void FGInterface::set_Sea_level_radius(double slr) {
-    sea_level_radius = slr;
-}
-
-void FGInterface::set_Runway_altitude(double ralt) {
-    runway_altitude = ralt;
-}
-
 void FGInterface::set_Static_pressure(double p) { static_pressure = p; }
 void FGInterface::set_Static_temperature(double T) { static_temperature = T; }
 void FGInterface::set_Density(double rho) { density = rho; }
