@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: atlas.cxx,v 1.6 2001/03/26 16:22:31 curt Exp $
+// $Id: atlas.cxx,v 1.4 2004/02/07 21:37:35 david Exp $
 
 
 #include <simgear/debug/logstream.hxx>
@@ -63,6 +63,9 @@ static char calc_atlas_cksum(char *sentence) {
 bool FGAtlas::gen_message() {
     // cout << "generating atlas message" << endl;
 
+    static SGPropertyNode *adf_freq
+        = fgGetNode("/radios/kr-87/outputs/selected-khz", true);
+
     char rmc[256], gga[256], patla[256];
     char rmc_sum[10], gga_sum[10], patla_sum[10];
     char dir;
@@ -126,11 +129,11 @@ bool FGAtlas::gen_message() {
     sprintf( gga_sum, "%02X", calc_atlas_cksum(gga) );
 
     sprintf( patla, "PATLA,%.2f,%.1f,%.2f,%.1f,%.0f",
-	     current_radiostack->get_nav1_freq(),
-	     current_radiostack->get_nav1_sel_radial(),
-	     current_radiostack->get_nav2_freq(),
-	     current_radiostack->get_nav2_sel_radial(),
-	     current_radiostack->get_adf_freq() );
+	     current_radiostack->get_navcom1()->get_nav_freq(),
+	     current_radiostack->get_navcom1()->get_nav_sel_radial(),
+	     current_radiostack->get_navcom1()->get_nav_freq(),
+	     current_radiostack->get_navcom1()->get_nav_sel_radial(),
+	     adf_freq->getDoubleValue() );
     sprintf( patla_sum, "%02X", calc_atlas_cksum(patla) );
 
     SG_LOG( SG_IO, SG_DEBUG, rmc );
@@ -160,7 +163,7 @@ bool FGAtlas::gen_message() {
     atlas_sentence += patla_sum;
     atlas_sentence += "\n";
 
-    cout << atlas_sentence;
+    //    cout << atlas_sentence;
 
     length = atlas_sentence.length();
     strncpy( buf, atlas_sentence.c_str(), length );
@@ -443,7 +446,7 @@ bool FGAtlas::parse_message() {
 	    string alt_units = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
-	    if ( alt_units != (string)"F" ) {
+	    if ( alt_units != "F" ) {
 		altitude *= SG_METER_TO_FEET;
 	    }
 
@@ -541,20 +544,20 @@ bool FGAtlas::process() {
     if ( get_direction() == SG_IO_OUT ) {
 	gen_message();
 	if ( ! io->write( buf, length ) ) {
-	    SG_LOG( SG_IO, SG_ALERT, "Error writing data." );
+	    SG_LOG( SG_IO, SG_WARN, "Error writing data." );
 	    return false;
 	}
     } else if ( get_direction() == SG_IO_IN ) {
 	if ( (length = io->readline( buf, FG_MAX_MSG_SIZE )) > 0 ) {
 	    parse_message();
 	} else {
-	    SG_LOG( SG_IO, SG_ALERT, "Error reading data." );
+	    SG_LOG( SG_IO, SG_WARN, "Error reading data." );
 	    return false;
 	}
 	if ( (length = io->readline( buf, FG_MAX_MSG_SIZE )) > 0 ) {
 	    parse_message();
 	} else {
-	    SG_LOG( SG_IO, SG_ALERT, "Error reading data." );
+	    SG_LOG( SG_IO, SG_WARN, "Error reading data." );
 	    return false;
 	}
     }
