@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-// $Id: beacon.cxx,v 1.1 2001/03/28 05:12:11 curt Exp $
+// $Id: beacon.cxx,v 1.3 2003/05/09 19:39:52 curt Exp $
 
 
 #include "beacon.hxx"
@@ -51,7 +51,7 @@ bool FGBeacon::init() {
 	ptr += INNER_DIT_LEN;
     }
 
-    inner = new FGSimpleSound( inner_buf, INNER_SIZE );
+    inner = new SGSimpleSound( inner_buf, INNER_SIZE );
 
     // Make middle marker beacon sound
     len= (int)(MIDDLE_DIT_LEN / 2.0 );
@@ -69,7 +69,7 @@ bool FGBeacon::init() {
     ptr += MIDDLE_DIT_LEN;
     memcpy( ptr, middle_dah, MIDDLE_DAH_LEN );
 
-    middle = new FGSimpleSound( middle_buf, MIDDLE_SIZE );
+    middle = new SGSimpleSound( middle_buf, MIDDLE_SIZE );
 
     // Make outer marker beacon sound
     len= (int)(OUTER_DAH_LEN * 3.0 / 4.0 );
@@ -82,7 +82,7 @@ bool FGBeacon::init() {
     ptr += OUTER_DAH_LEN;
     memcpy( ptr, outer_dah, OUTER_DAH_LEN );
 
-    outer = new FGSimpleSound( outer_buf, OUTER_SIZE );
+    outer = new SGSimpleSound( outer_buf, OUTER_SIZE );
 
     return true;
 }
